@@ -49,6 +49,12 @@ Core::Id ClearCaseControl::id() const
     return Constants::VCS_ID_CLEARCASE;
 }
 
+bool ClearCaseControl::isVcsFileOrDirectory(const Utils::FileName &fileName) const
+{
+    Q_UNUSED(fileName);
+    return false; // ClearCase has no files/directories littering the sources
+}
+
 bool ClearCaseControl::isConfigured() const
 {
 #ifdef WITH_TESTS
