@@ -26,15 +26,18 @@
 #include "settingsdialog.h"
 
 #include <coreplugin/icore.h>
+#include <coreplugin/dialogs/ioptionspage.h>
+#include <coreplugin/iwizardfactory.h>
 
-#include <extensionsystem/pluginmanager.h>
 #include <utils/algorithm.h>
 #include <utils/hostosinfo.h>
 #include <utils/fancylineedit.h>
 #include <utils/qtcassert.h>
 
 #include <QApplication>
+#include <QDialog>
 #include <QDialogButtonBox>
+#include <QEventLoop>
 #include <QGridLayout>
 #include <QHBoxLayout>
 #include <QIcon>
@@ -45,6 +48,7 @@
 #include <QResizeEvent>
 #include <QScrollArea>
 #include <QScrollBar>
+#include <QSet>
 #include <QSettings>
 #include <QSortFilterProxyModel>
 #include <QSpacerItem>
@@ -52,14 +56,17 @@
 #include <QStyle>
 #include <QStyledItemDelegate>
 
+const int kInitialWidth = 750;
+const int kInitialHeight = 450;
+const int kMaxMinimumWidth = 250;
+const int kMaxMinimumHeight = 250;
+
 static const char pageKeyC[] = "General/LastPreferencePage";
 const int categoryIconSize = 24;
 
 namespace Core {
 namespace Internal {
 
-static QPointer<SettingsDialog> m_instance = 0;
-
 bool optionsPageLessThan(const IOptionsPage *p1, const IOptionsPage *p2)
 {
     if (p1->category() != p2->category())
@@ -69,8 +76,8 @@ bool optionsPageLessThan(const IOptionsPage *p1, const IOptionsPage *p2)
 
 static inline QList<IOptionsPage*> sortedOptionsPages()
 {
-    QList<IOptionsPage*> rc = ExtensionSystem::PluginManager::getObjects<IOptionsPage>();
-    qStableSort(rc.begin(), rc.end(), optionsPageLessThan);
+    QList<IOptionsPage*> rc = IOptionsPage::allOptionsPages();
+    std::stable_sort(rc.begin(), rc.end(), optionsPageLessThan);
     return rc;
 }
 
@@ -79,8 +86,6 @@ static inline QList<IOptionsPage*> sortedOptionsPages()
 class Category
 {
 public:
-    Category() : index(-1), providerPagesCreated(false) { }
-
     bool findPageById(const Id id, int *pageIndex) const
     {
         *pageIndex = Utils::indexOf(pages, Utils::equal(&IOptionsPage::id, id));
@@ -88,20 +93,20 @@ public:
     }
 
     Id id;
-    int index;
+    int index = -1;
     QString displayName;
     QIcon icon;
     QList<IOptionsPage *> pages;
     QList<IOptionsPageProvider *> providers;
-    bool providerPagesCreated;
-    QTabWidget *tabWidget;
+    bool providerPagesCreated = false;
+    QTabWidget *tabWidget = nullptr;
 };
 
 class CategoryModel : public QAbstractListModel
 {
 public:
-    CategoryModel(QObject *parent = 0);
-    ~CategoryModel();
+    CategoryModel();
+    ~CategoryModel() override;
 
     int rowCount(const QModelIndex &parent = QModelIndex()) const override;
     QVariant data(const QModelIndex &index, int role = Qt::DisplayRole) const override;
@@ -119,8 +124,7 @@ private:
     QIcon m_emptyIcon;
 };
 
-CategoryModel::CategoryModel(QObject *parent)
-    : QAbstractListModel(parent)
+CategoryModel::CategoryModel()
 {
     QPixmap empty(categoryIconSize, categoryIconSize);
     empty.fill(Qt::transparent);
@@ -173,7 +177,7 @@ void CategoryModel::setPages(const QList<IOptionsPage*> &pages,
         if (!category) {
             category = new Category;
             category->id = categoryId;
-            category->tabWidget = 0;
+            category->tabWidget = nullptr;
             category->index = -1;
             m_categories.append(category);
         }
@@ -190,7 +194,7 @@ void CategoryModel::setPages(const QList<IOptionsPage*> &pages,
         if (!category) {
             category = new Category;
             category->id = categoryId;
-            category->tabWidget = 0;
+            category->tabWidget = nullptr;
             category->index = -1;
             m_categories.append(category);
         }
@@ -222,7 +226,7 @@ void CategoryModel::ensurePages(Category *category)
 
         category->pages += createdPages;
         category->providerPagesCreated = true;
-        qStableSort(category->pages.begin(), category->pages.end(), optionsPageLessThan);
+        std::stable_sort(category->pages.begin(), category->pages.end(), optionsPageLessThan);
     }
 }
 
@@ -234,7 +238,7 @@ Category *CategoryModel::findCategoryById(Id id)
             return category;
     }
 
-    return 0;
+    return nullptr;
 }
 
 // ----------- Category filter model
@@ -246,9 +250,7 @@ Category *CategoryModel::findCategoryById(Id id)
 class CategoryFilterModel : public QSortFilterProxyModel
 {
 public:
-    explicit CategoryFilterModel(QObject *parent = 0)
-        : QSortFilterProxyModel(parent)
-    {}
+    CategoryFilterModel() = default;
 
 protected:
     bool filterAcceptsRow(int sourceRow, const QModelIndex &sourceParent) const override;
@@ -263,7 +265,7 @@ bool CategoryFilterModel::filterAcceptsRow(int sourceRow, const QModelIndex &sou
     const QString pattern = filterRegExp().pattern();
     const CategoryModel *cm = static_cast<CategoryModel*>(sourceModel());
     const Category *category = cm->categories().at(sourceRow);
-    foreach (const IOptionsPage *page, category->pages) {
+    for (const IOptionsPage *page : category->pages) {
         if (page->displayCategory().contains(pattern, Qt::CaseInsensitive)
                 || page->displayName().contains(pattern, Qt::CaseInsensitive)
                 || page->matches(pattern))
@@ -271,7 +273,7 @@ bool CategoryFilterModel::filterAcceptsRow(int sourceRow, const QModelIndex &sou
     }
 
     if (!category->providerPagesCreated) {
-        foreach (const IOptionsPageProvider *provider, category->providers) {
+        for (const IOptionsPageProvider *provider : category->providers) {
             if (provider->matches(pattern))
                 return true;
         }
@@ -282,12 +284,12 @@ bool CategoryFilterModel::filterAcceptsRow(int sourceRow, const QModelIndex &sou
 
 // ----------- Category list view
 
-
 class CategoryListViewDelegate : public QStyledItemDelegate
 {
 public:
-    CategoryListViewDelegate(QObject *parent) : QStyledItemDelegate(parent) {}
-    QSize sizeHint(const QStyleOptionViewItem &option, const QModelIndex &index) const
+    explicit CategoryListViewDelegate(QObject *parent) : QStyledItemDelegate(parent) {}
+
+    QSize sizeHint(const QStyleOptionViewItem &option, const QModelIndex &index) const override
     {
         QSize size = QStyledItemDelegate::sizeHint(option, index);
         size.setHeight(qMax(size.height(), 32));
@@ -302,26 +304,25 @@ public:
 class CategoryListView : public QListView
 {
 public:
-    CategoryListView(QWidget *parent = 0) : QListView(parent)
+    CategoryListView()
     {
         setSizePolicy(QSizePolicy::MinimumExpanding, QSizePolicy::Expanding);
         setItemDelegate(new CategoryListViewDelegate(this));
         setHorizontalScrollBarPolicy(Qt::ScrollBarAlwaysOff);
     }
 
-    virtual QSize sizeHint() const
+    QSize sizeHint() const final
     {
         int width = sizeHintForColumn(0) + frameWidth() * 2 + 5;
-        width += verticalScrollBar()->width();
+        width += verticalScrollBar()->sizeHint().width();
         return QSize(width, 100);
     }
 
     // QListView installs a event filter on its scrollbars
-    virtual bool eventFilter(QObject *obj, QEvent *event)
+    bool eventFilter(QObject *obj, QEvent *event) final
     {
         if (obj == verticalScrollBar()
-                && (event->type() == QEvent::Show
-                    || event->type() == QEvent::Hide))
+                && (event->type() == QEvent::Show || event->type() == QEvent::Hide))
             updateGeometry();
         return QListView::eventFilter(obj, event);
     }
@@ -332,7 +333,7 @@ public:
 class SmartScrollArea : public QScrollArea
 {
 public:
-    SmartScrollArea(QWidget *parent = 0)
+    explicit SmartScrollArea(QWidget *parent)
         : QScrollArea(parent)
     {
         setFrameStyle(QFrame::NoFrame | QFrame::Plain);
@@ -340,7 +341,7 @@ public:
         setWidgetResizable(true);
     }
 private:
-    void resizeEvent(QResizeEvent *event) override
+    void resizeEvent(QResizeEvent *event) final
     {
         QWidget *inner = widget();
         if (inner) {
@@ -357,7 +358,8 @@ private:
         QScrollArea::resizeEvent(event);
     }
 
-    QSize minimumSizeHint() const override {
+    QSize minimumSizeHint() const final
+    {
         QWidget *inner = widget();
         if (inner) {
             int fw = frameWidth() * 2;
@@ -365,14 +367,15 @@ private:
             QSize minSize = inner->minimumSizeHint();
             minSize += QSize(fw, fw);
             minSize += QSize(scrollBarWidth(), 0);
-            minSize.setHeight(qMin(minSize.height(), 450));
-            minSize.setWidth(qMin(minSize.width(), 810));
+            minSize.setWidth(qMin(minSize.width(), kMaxMinimumWidth));
+            minSize.setHeight(qMin(minSize.height(), kMaxMinimumHeight));
             return minSize;
         }
         return QSize(0, 0);
     }
 
-    bool event(QEvent *event) override {
+    bool event(QEvent *event) final
+    {
         if (event->type() == QEvent::LayoutRequest)
             updateGeometry();
         return QScrollArea::event(event);
@@ -388,39 +391,77 @@ private:
     }
 };
 
-
 // ----------- SettingsDialog
 
+class SettingsDialog : public QDialog
+{
+public:
+    explicit SettingsDialog(QWidget *parent);
+
+    void showPage(Id pageId);
+    bool execDialog();
+
+private:
+    // Make sure the settings dialog starts up as small as possible.
+    QSize sizeHint() const final { return minimumSize(); }
+
+    void done(int) final;
+    void accept() final;
+    void reject() final;
+
+    void apply();
+    void currentChanged(const QModelIndex &current);
+    void currentTabChanged(int);
+    void filter(const QString &text);
+
+    void createGui();
+    void showCategory(int index);
+    void updateEnabledTabs(Category *category, const QString &searchText);
+    void ensureCategoryWidget(Category *category);
+    void disconnectTabWidgets();
+
+    const QList<IOptionsPage *> m_pages;
+
+    QSet<IOptionsPage *> m_visitedPages;
+    CategoryFilterModel m_proxyModel;
+    CategoryModel m_model;
+    Id m_currentCategory;
+    Id m_currentPage;
+    QStackedLayout *m_stackedLayout;
+    Utils::FancyLineEdit *m_filterLineEdit;
+    QListView *m_categoryList;
+    QLabel *m_headerLabel;
+    std::vector<QEventLoop *> m_eventLoops;
+    bool m_running = false;
+    bool m_applied = false;
+    bool m_finished = false;
+};
+
+static QPointer<SettingsDialog> m_instance = nullptr;
+
 SettingsDialog::SettingsDialog(QWidget *parent) :
     QDialog(parent),
     m_pages(sortedOptionsPages()),
-    m_proxyModel(new CategoryFilterModel(this)),
-    m_model(new CategoryModel(this)),
     m_stackedLayout(new QStackedLayout),
     m_filterLineEdit(new Utils::FancyLineEdit),
     m_categoryList(new CategoryListView),
-    m_headerLabel(new QLabel),
-    m_running(false),
-    m_applied(false),
-    m_finished(false)
+    m_headerLabel(new QLabel)
 {
-    m_applied = false;
     m_filterLineEdit->setFiltering(true);
 
     createGui();
     setWindowFlags(windowFlags() & ~Qt::WindowContextHelpButtonHint);
     if (Utils::HostOsInfo::isMacHost())
-        setWindowTitle(tr("Preferences"));
+        setWindowTitle(QCoreApplication::translate("Core::Internal::SettingsDialog", "Preferences"));
     else
-        setWindowTitle(tr("Options"));
+        setWindowTitle(QCoreApplication::translate("Core::Internal::SettingsDialog", "Options"));
 
-    m_model->setPages(m_pages,
-        ExtensionSystem::PluginManager::getObjects<IOptionsPageProvider>());
+    m_model.setPages(m_pages, IOptionsPageProvider::allOptionsPagesProviders());
 
-    m_proxyModel->setSourceModel(m_model);
-    m_proxyModel->setFilterCaseSensitivity(Qt::CaseInsensitive);
+    m_proxyModel.setSourceModel(&m_model);
+    m_proxyModel.setFilterCaseSensitivity(Qt::CaseInsensitive);
     m_categoryList->setIconSize(QSize(categoryIconSize, categoryIconSize));
-    m_categoryList->setModel(m_proxyModel);
+    m_categoryList->setModel(&m_proxyModel);
     m_categoryList->setSelectionMode(QAbstractItemView::SingleSelection);
     m_categoryList->setVerticalScrollMode(QAbstractItemView::ScrollPerPixel);
 
@@ -430,7 +471,7 @@ SettingsDialog::SettingsDialog(QWidget *parent) :
     // The order of the slot connection matters here, the filter slot
     // opens the matching page after the model has filtered.
     connect(m_filterLineEdit, &Utils::FancyLineEdit::filterChanged,
-            m_proxyModel, &QSortFilterProxyModel::setFilterFixedString);
+            &m_proxyModel, &QSortFilterProxyModel::setFilterFixedString);
     connect(m_filterLineEdit, &Utils::FancyLineEdit::filterChanged,
             this, &SettingsDialog::filter);
     m_categoryList->setFocus();
@@ -448,7 +489,7 @@ void SettingsDialog::showPage(const Id pageId)
     int initialCategoryIndex = -1;
     int initialPageIndex = -1;
 
-    const QList<Category*> &categories = m_model->categories();
+    const QList<Category *> &categories = m_model.categories();
     if (initialPageId.isValid()) {
         // First try categories without lazy items.
         for (int i = 0; i < categories.size(); ++i) {
@@ -480,10 +521,16 @@ void SettingsDialog::showPage(const Id pageId)
         return; // Unknown settings page, probably due to missing plugin.
 
     if (initialCategoryIndex != -1) {
-        const QModelIndex modelIndex = m_proxyModel->mapFromSource(m_model->index(initialCategoryIndex));
+        QModelIndex modelIndex = m_proxyModel.mapFromSource(m_model.index(initialCategoryIndex));
+        if (!modelIndex.isValid()) { // filtered out, so clear filter first
+            m_filterLineEdit->setText(QString());
+            modelIndex = m_proxyModel.mapFromSource(m_model.index(initialCategoryIndex));
+        }
         m_categoryList->setCurrentIndex(modelIndex);
-        if (initialPageIndex != -1)
-            categories.at(initialCategoryIndex)->tabWidget->setCurrentIndex(initialPageIndex);
+        if (initialPageIndex != -1) {
+            if (QTC_GUARD(categories.at(initialCategoryIndex)->tabWidget))
+                categories.at(initialCategoryIndex)->tabWidget->setCurrentIndex(initialPageIndex);
+        }
     }
 }
 
@@ -498,13 +545,14 @@ void SettingsDialog::createGui()
         headerLabelFont.setPointSize(pointSize + 2);
     m_headerLabel->setFont(headerLabelFont);
 
-    QHBoxLayout *headerHLayout = new QHBoxLayout;
-    const int leftMargin = qApp->style()->pixelMetric(QStyle::PM_LayoutLeftMargin);
+    auto headerHLayout = new QHBoxLayout;
+    const int leftMargin = QApplication::style()->pixelMetric(QStyle::PM_LayoutLeftMargin);
     headerHLayout->addSpacerItem(new QSpacerItem(leftMargin, 0, QSizePolicy::Fixed, QSizePolicy::Ignored));
     headerHLayout->addWidget(m_headerLabel);
 
     m_stackedLayout->setMargin(0);
-    m_stackedLayout->addWidget(new QWidget(this)); // no category selected, for example when filtering
+    QWidget *emptyWidget = new QWidget(this);
+    m_stackedLayout->addWidget(emptyWidget); // no category selected, for example when filtering
 
     QDialogButtonBox *buttonBox = new QDialogButtonBox(QDialogButtonBox::Ok |
                                                        QDialogButtonBox::Apply |
@@ -515,7 +563,7 @@ void SettingsDialog::createGui()
     connect(buttonBox, &QDialogButtonBox::accepted, this, &SettingsDialog::accept);
     connect(buttonBox, &QDialogButtonBox::rejected, this, &SettingsDialog::reject);
 
-    QGridLayout *mainGridLayout = new QGridLayout;
+    auto mainGridLayout = new QGridLayout;
     mainGridLayout->addWidget(m_filterLineEdit, 0, 0, 1, 1);
     mainGridLayout->addLayout(headerHLayout,    0, 1, 1, 1);
     mainGridLayout->addWidget(m_categoryList,   1, 0, 1, 1);
@@ -529,13 +577,9 @@ void SettingsDialog::createGui()
     mainGridLayout->setSizeConstraint(QLayout::SetMinimumSize);
 }
 
-SettingsDialog::~SettingsDialog()
-{
-}
-
 void SettingsDialog::showCategory(int index)
 {
-    Category *category = m_model->categories().at(index);
+    Category *category = m_model.categories().at(index);
     ensureCategoryWidget(category);
     // Update current category and page
     m_currentCategory = category->id;
@@ -554,15 +598,16 @@ void SettingsDialog::showCategory(int index)
 
 void SettingsDialog::ensureCategoryWidget(Category *category)
 {
-    if (category->tabWidget != 0)
+    if (category->tabWidget)
         return;
 
-    m_model->ensurePages(category);
-    QTabWidget *tabWidget = new QTabWidget;
-    for (int j = 0; j < category->pages.size(); ++j) {
-        IOptionsPage *page = category->pages.at(j);
+    m_model.ensurePages(category);
+    auto tabWidget = new QTabWidget;
+    tabWidget->tabBar()->setObjectName("qc_settings_main_tabbar"); // easier lookup in Squish
+    for (IOptionsPage *page : category->pages) {
         QWidget *widget = page->widget();
-        SmartScrollArea *ssa = new SmartScrollArea(this);
+        ICore::setupScreenShooter(page->displayName(), widget);
+        auto ssa = new SmartScrollArea(this);
         ssa->setWidget(widget);
         widget->setAutoFillBackground(false);
         tabWidget->addTab(ssa, page->displayName());
@@ -577,7 +622,7 @@ void SettingsDialog::ensureCategoryWidget(Category *category)
 
 void SettingsDialog::disconnectTabWidgets()
 {
-    foreach (Category *category, m_model->categories()) {
+    for (Category *category : m_model.categories()) {
         if (category->tabWidget)
             disconnect(category->tabWidget, &QTabWidget::currentChanged,
                        this, &SettingsDialog::currentTabChanged);
@@ -607,7 +652,7 @@ void SettingsDialog::updateEnabledTabs(Category *category, const QString &search
 void SettingsDialog::currentChanged(const QModelIndex &current)
 {
     if (current.isValid()) {
-        showCategory(m_proxyModel->mapToSource(current).row());
+        showCategory(m_proxyModel.mapToSource(current).row());
     } else {
         m_stackedLayout->setCurrentIndex(0);
         m_headerLabel->clear();
@@ -619,12 +664,12 @@ void SettingsDialog::currentTabChanged(int index)
     if (index == -1)
         return;
 
-    const QModelIndex modelIndex = m_proxyModel->mapToSource(m_categoryList->currentIndex());
+    const QModelIndex modelIndex = m_proxyModel.mapToSource(m_categoryList->currentIndex());
     if (!modelIndex.isValid())
         return;
 
     // Remember the current tab and mark it as visited
-    const Category *category = m_model->categories().at(modelIndex.row());
+    const Category *category = m_model.categories().at(modelIndex.row());
     IOptionsPage *page = category->pages.at(index);
     m_currentPage = page->id();
     m_visitedPages.insert(page);
@@ -633,14 +678,14 @@ void SettingsDialog::currentTabChanged(int index)
 void SettingsDialog::filter(const QString &text)
 {
     // When there is no current index, select the first one when possible
-    if (!m_categoryList->currentIndex().isValid() && m_model->rowCount() > 0)
-        m_categoryList->setCurrentIndex(m_proxyModel->index(0, 0));
+    if (!m_categoryList->currentIndex().isValid() && m_model.rowCount() > 0)
+        m_categoryList->setCurrentIndex(m_proxyModel.index(0, 0));
 
-    const QModelIndex currentIndex = m_proxyModel->mapToSource(m_categoryList->currentIndex());
+    const QModelIndex currentIndex = m_proxyModel.mapToSource(m_categoryList->currentIndex());
     if (!currentIndex.isValid())
         return;
 
-    Category *category = m_model->categories().at(currentIndex.row());
+    Category *category = m_model.categories().at(currentIndex.row());
     updateEnabledTabs(category, text);
 }
 
@@ -683,33 +728,14 @@ void SettingsDialog::done(int val)
 
     ICore::saveSettings(); // save all settings
 
-    // exit all additional event loops, see comment in execDialog()
-    QListIterator<QEventLoop *> it(m_eventLoops);
-    it.toBack();
-    while (it.hasPrevious()) {
-        QEventLoop *loop = it.previous();
-        loop->exit();
-    }
+    // exit event loops in reverse order of addition
+    for (QEventLoop *eventLoop : m_eventLoops)
+        eventLoop->exit();
+    m_eventLoops.clear();
 
     QDialog::done(val);
 }
 
-/**
- * Override to make sure the settings dialog starts up as small as possible.
- */
-QSize SettingsDialog::sizeHint() const
-{
-    return minimumSize();
-}
-
-SettingsDialog *SettingsDialog::getSettingsDialog(QWidget *parent, Id initialPage)
-{
-    if (!m_instance)
-        m_instance = new SettingsDialog(parent);
-    m_instance->showPage(initialPage);
-    return m_instance;
-}
-
 bool SettingsDialog::execDialog()
 {
     if (!m_running) {
@@ -718,9 +744,11 @@ bool SettingsDialog::execDialog()
         static const QLatin1String kPreferenceDialogSize("Core/PreferenceDialogSize");
         if (ICore::settings()->contains(kPreferenceDialogSize))
             resize(ICore::settings()->value(kPreferenceDialogSize).toSize());
+        else
+            resize(kInitialWidth, kInitialHeight);
         exec();
         m_running = false;
-        m_instance = 0;
+        m_instance = nullptr;
         ICore::settings()->setValue(kPreferenceDialogSize, size());
         // make sure that the current "single" instance is deleted
         // we can't delete right away, since we still access the m_applied member
@@ -734,12 +762,25 @@ bool SettingsDialog::execDialog()
         // a break point it will complain about missing helper, and offer the
         // option to open the settings dialog.
         // Keep the UI running by creating another event loop.
-        QEventLoop *loop = new QEventLoop(this);
-        m_eventLoops.append(loop);
-        loop->exec();
+        QEventLoop eventLoop;
+        m_eventLoops.emplace(m_eventLoops.begin(), &eventLoop);
+        eventLoop.exec();
+        QTC_ASSERT(m_eventLoops.empty(), return m_applied;);
     }
     return m_applied;
 }
 
+bool executeSettingsDialog(QWidget *parent, Id initialPage)
+{
+    // Make sure all wizards are there when the user might access the keyboard shortcuts:
+    (void) IWizardFactory::allWizardFactories();
+
+    if (!m_instance)
+        m_instance = new SettingsDialog(parent);
+
+    m_instance->showPage(initialPage);
+    return m_instance->execDialog();
+}
+
 } // namespace Internal
 } // namespace Core
