@@ -23,8 +23,7 @@
 **
 ****************************************************************************/
 
-#ifndef EDITORMANAGER_H
-#define EDITORMANAGER_H
+#pragma once
 
 #include "../core_global.h"
 #include "../idocument.h"
@@ -42,15 +41,11 @@ namespace Utils { class MimeType; }
 
 namespace Core {
 
-class IContext;
 class IEditor;
 class IEditorFactory;
 class IExternalEditor;
 class IDocument;
-class IMode;
-class IVersionControl;
-
-class EditorToolBar;
+class SearchResultItem;
 
 enum MakeWritableResult {
     OpenedWithVersionControl,
@@ -60,26 +55,19 @@ enum MakeWritableResult {
 };
 
 namespace Internal {
-class EditorClosingCoreListener;
 class EditorManagerPrivate;
-class EditorView;
 class MainWindow;
-class OpenEditorsViewFactory;
-class OpenEditorsWindow;
-class SplitterOrView;
 } // namespace Internal
 
 class CORE_EXPORT EditorManagerPlaceHolder : public QWidget
 {
     Q_OBJECT
 public:
-    explicit EditorManagerPlaceHolder(IMode *mode, QWidget *parent = 0);
-    ~EditorManagerPlaceHolder();
+    explicit EditorManagerPlaceHolder(QWidget *parent = nullptr);
+    ~EditorManagerPlaceHolder() final;
 
-private:
-    void currentModeChanged(IMode *);
-
-    IMode *m_mode;
+protected:
+    void showEvent(QShowEvent *event) override;
 };
 
 class CORE_EXPORT EditorManager : public QObject
@@ -87,9 +75,7 @@ class CORE_EXPORT EditorManager : public QObject
     Q_OBJECT
 
 public:
-    typedef QList<IEditorFactory *> EditorFactoryList;
-    typedef QList<IExternalEditor *> ExternalEditorList;
-    typedef std::function<QString (const QString &)> WindowTitleHandler;
+    using WindowTitleHandler = std::function<QString (const QString &)>;
 
     static EditorManager *instance();
 
@@ -100,7 +86,9 @@ public:
         DoNotMakeVisible = 4,
         CanContainLineAndColumnNumber = 8,
         OpenInOtherSplit = 16,
-        DoNotSwitchToDesignMode = 32
+        DoNotSwitchToDesignMode = 32,
+        DoNotSwitchToEditMode = 64,
+        SwitchSplitIfAlreadyVisible = 128
     };
     Q_DECLARE_FLAGS(OpenEditorFlags, OpenEditorFlag)
 
@@ -112,14 +100,17 @@ public:
     };
     static FilePathInfo splitLineAndColumnNumber(const QString &filePath);
     static IEditor *openEditor(const QString &fileName, Id editorId = Id(),
-        OpenEditorFlags flags = NoFlags, bool *newEditor = 0);
+        OpenEditorFlags flags = NoFlags, bool *newEditor = nullptr);
     static IEditor *openEditorAt(const QString &fileName,  int line, int column = 0,
                                  Id editorId = Id(), OpenEditorFlags flags = NoFlags,
-                                 bool *newEditor = 0);
-    static IEditor *openEditorWithContents(Id editorId, QString *titlePattern = 0,
+                                 bool *newEditor = nullptr);
+    static void openEditorAtSearchResult(const SearchResultItem &item, OpenEditorFlags flags = NoFlags);
+    static IEditor *openEditorWithContents(Id editorId, QString *titlePattern = nullptr,
                                            const QByteArray &contents = QByteArray(),
                                            const QString &uniqueId = QString(),
                                            OpenEditorFlags flags = NoFlags);
+    static bool skipOpeningBigTextFile(const QString &filePath);
+    static void clearUniqueId(IDocument *document);
 
     static bool openExternalEditor(const QString &fileName, Id editorId);
     static void addCloseEditorListener(const std::function<bool(IEditor *)> &listener);
@@ -130,9 +121,9 @@ public:
     static IEditor *currentEditor();
     static QList<IEditor *> visibleEditors();
 
-    static void activateEditor(IEditor *editor, OpenEditorFlags flags = 0);
-    static void activateEditorForEntry(DocumentModel::Entry *entry, OpenEditorFlags flags = 0);
-    static IEditor *activateEditorForDocument(IDocument *document, OpenEditorFlags flags = 0);
+    static void activateEditor(IEditor *editor, OpenEditorFlags flags = NoFlags);
+    static void activateEditorForEntry(DocumentModel::Entry *entry, OpenEditorFlags flags = NoFlags);
+    static IEditor *activateEditorForDocument(IDocument *document, OpenEditorFlags flags = NoFlags);
 
     static bool closeDocument(IDocument *document, bool askAboutModifiedEditors = true);
     static bool closeDocuments(const QList<IDocument *> &documents, bool askAboutModifiedEditors = true);
@@ -152,14 +143,12 @@ public:
     static bool hasSplitter();
 
     static void showEditorStatusBar(const QString &id,
-                           const QString &infoText,
-                           const QString &buttonText = QString(),
-                           QObject *object = 0, const char *member = 0);
+                                    const QString &infoText,
+                                    const QString &buttonText = QString(),
+                                    QObject *object = nullptr,
+                                    const std::function<void()> &function = nullptr);
     static void hideEditorStatusBar(const QString &id);
 
-    static EditorFactoryList editorFactories(const Utils::MimeType &mimeType, bool bestMatchOnly = true);
-    static ExternalEditorList externalEditors(const Utils::MimeType &mimeType, bool bestMatchOnly = true);
-
     static bool isAutoSaveFile(const QString &fileName);
 
     static QTextCodec *defaultTextCodec();
@@ -167,10 +156,11 @@ public:
     static qint64 maxTextFileSize();
 
     static void setWindowTitleAdditionHandler(WindowTitleHandler handler);
+    static void setSessionTitleHandler(WindowTitleHandler handler);
     static void setWindowTitleVcsTopicHandler(WindowTitleHandler handler);
 
     static void addSaveAndCloseEditorActions(QMenu *contextMenu, DocumentModel::Entry *entry,
-                                             IEditor *editor = 0);
+                                             IEditor *editor = nullptr);
     static void addNativeDirAndOpenWithActions(QMenu *contextMenu, DocumentModel::Entry *entry);
     static void populateOpenWithMenu(QMenu *menu, const QString &fileName);
 
@@ -181,11 +171,17 @@ public: // for tests
 signals:
     void currentEditorChanged(Core::IEditor *editor);
     void currentDocumentStateChanged();
+    void documentStateChanged(Core::IDocument *document);
     void editorCreated(Core::IEditor *editor, const QString &fileName);
     void editorOpened(Core::IEditor *editor);
     void editorAboutToClose(Core::IEditor *editor);
     void editorsClosed(QList<Core::IEditor *> editors);
     void findOnFileSystemRequest(const QString &path);
+    void openFileProperties(const Utils::FileName &path);
+    void aboutToSave(IDocument *document);
+    void saved(IDocument *document);
+    void autoSaved();
+    void currentEditorAboutToChange(Core::IEditor *editor);
 
 public slots:
     static void saveDocument();
@@ -202,7 +198,7 @@ public slots:
 
 private:
     explicit EditorManager(QObject *parent);
-    ~EditorManager();
+    ~EditorManager() override;
 
     friend class Core::Internal::MainWindow;
 };
@@ -210,5 +206,3 @@ private:
 } // namespace Core
 
 Q_DECLARE_OPERATORS_FOR_FLAGS(Core::EditorManager::OpenEditorFlags)
-
-#endif // EDITORMANAGER_H
