@@ -27,11 +27,16 @@
 
 #include <coreplugin/core_global.h>
 
+#include <utils/mimetypes/mimetype.h>
+
 #include <QObject>
 
 namespace Core {
 
 class Id;
+class IExternalEditor;
+
+using ExternalEditorList = QList<IExternalEditor *>;
 
 class CORE_EXPORT IExternalEditor : public QObject
 {
@@ -39,9 +44,10 @@ class CORE_EXPORT IExternalEditor : public QObject
 
 public:
     explicit IExternalEditor(QObject *parent = nullptr);
-    ~IExternalEditor();
+    ~IExternalEditor() override;
 
-    static const QList<IExternalEditor *> allExternalEditors();
+    static const ExternalEditorList allExternalEditors();
+    static const ExternalEditorList externalEditors(const Utils::MimeType &mimeType);
 
     virtual QStringList mimeTypes() const = 0;
     virtual Id id() const = 0;
