@@ -38,11 +38,11 @@
 #include <texteditor/codeassist/genericproposal.h>
 #include <texteditor/codeassist/ifunctionhintproposalmodel.h>
 #include <texteditor/codeassist/functionhintproposal.h>
-#include <texteditor/convenience.h>
 #include <texteditor/snippets/snippet.h>
 #include <texteditor/texteditorsettings.h>
 #include <texteditor/completionsettings.h>
 
+#include <utils/textutils.h>
 #include <utils/mimetypes/mimedatabase.h>
 #include <utils/qtcassert.h>
 
@@ -70,7 +70,7 @@ namespace Internal {
 
 struct CompleteFunctionDeclaration
 {
-    explicit CompleteFunctionDeclaration(Function *f = 0)
+    explicit CompleteFunctionDeclaration(Function *f = nullptr)
         : function(f)
     {}
 
@@ -83,10 +83,7 @@ struct CompleteFunctionDeclaration
 class CppAssistProposalItem final : public AssistProposalItem
 {
 public:
-    CppAssistProposalItem() :
-        m_isOverloaded(false) {}
-
-    ~CppAssistProposalItem() Q_DECL_NOEXCEPT {}
+    ~CppAssistProposalItem() noexcept override = default;
     bool prematurelyApplies(const QChar &c) const override;
     void applyContextualContent(TextDocumentManipulatorInterface &manipulator, int basePosition) const override;
 
@@ -101,9 +98,9 @@ public:
 
 private:
     QSharedPointer<TypeOfExpression> m_typeOfExpression;
-    unsigned m_completionOperator;
+    unsigned m_completionOperator = T_EOF_SYMBOL;
     mutable QChar m_typedChar;
-    bool m_isOverloaded;
+    bool m_isOverloaded = false;
 };
 
 } // Internal
@@ -123,7 +120,7 @@ AssistProposalItemInterface *CppAssistProposalModel::proposalItem(int index) con
 {
     AssistProposalItemInterface *item = GenericProposalModel::proposalItem(index);
     if (!item->isSnippet()) {
-        CppAssistProposalItem *cppItem = static_cast<CppAssistProposalItem *>(item);
+        auto cppItem = static_cast<CppAssistProposalItem *>(item);
         cppItem->keepCompletionOperator(m_completionOperator);
         cppItem->keepTypeOfExpression(m_typeOfExpression);
     }
@@ -194,7 +191,7 @@ quint64 CppAssistProposalItem::hash() const
 
 void CppAssistProposalItem::applyContextualContent(TextDocumentManipulatorInterface &manipulator, int basePosition) const
 {
-    Symbol *symbol = 0;
+    Symbol *symbol = nullptr;
 
     if (data().isValid())
         symbol = data().value<Symbol *>();
@@ -312,28 +309,20 @@ void CppAssistProposalItem::applyContextualContent(TextDocumentManipulatorInterf
             --cursorOffset;
     }
 
-    // Determine the length of characters that should just be kept on the editor, but do
-    // not consider content that ends as an identifier (which could be undesired).
-    const int lineEnd = manipulator.positionAt(EndOfLinePosition);
-    const QString inEditor = manipulator.textAt(manipulator.currentPosition(),
-                                                lineEnd - manipulator.currentPosition());
-    int preserveLength = 0;
-    if (!inEditor.isEmpty()) {
-        preserveLength = toInsert.length() - (manipulator.currentPosition() - basePosition);
-        const int inEditorLength = inEditor.length();
-        while (preserveLength > 0) {
-            if (inEditor.startsWith(toInsert.right(preserveLength))
-                    && (inEditorLength == preserveLength
-                        || !CppTools::isValidIdentifierChar(inEditor.at(preserveLength)))) {
-                break;
-            }
-            --preserveLength;
-        }
+    // Avoid inserting characters that are already there
+    int currentPosition = manipulator.currentPosition();
+    QTextCursor cursor = manipulator.textCursorAt(basePosition);
+    cursor.movePosition(QTextCursor::EndOfWord);
+    const QString textAfterCursor = manipulator.textAt(currentPosition,
+                                                       cursor.position() - currentPosition);
+    if (toInsert != textAfterCursor
+            && toInsert.indexOf(textAfterCursor, currentPosition - basePosition) >= 0) {
+        currentPosition = cursor.position();
     }
 
     for (int i = 0; i < extraChars.length(); ++i) {
         const QChar a = extraChars.at(i);
-        const QChar b = manipulator.characterAt(manipulator.currentPosition() + i + preserveLength);
+        const QChar b = manipulator.characterAt(currentPosition + i);
         if (a == b)
             ++extraLength;
         else
@@ -343,8 +332,9 @@ void CppAssistProposalItem::applyContextualContent(TextDocumentManipulatorInterf
     toInsert += extraChars;
 
     // Insert the remainder of the name
-    const int length = manipulator.currentPosition() - basePosition + preserveLength + extraLength;
+    const int length = currentPosition - basePosition + extraLength;
     manipulator.replace(basePosition, length, toInsert);
+    manipulator.setCursorPosition(basePosition + toInsert.length());
     if (cursorOffset)
         manipulator.setCursorPosition(manipulator.currentPosition() + cursorOffset);
     if (setAutoCompleteSkipPos)
@@ -357,7 +347,7 @@ void CppAssistProposalItem::applyContextualContent(TextDocumentManipulatorInterf
 class CppFunctionHintModel : public IFunctionHintProposalModel
 {
 public:
-    CppFunctionHintModel(QList<Function *> functionSymbols,
+    CppFunctionHintModel(const QList<Function *> &functionSymbols,
                          const QSharedPointer<TypeOfExpression> &typeOfExp)
         : m_functionSymbols(functionSymbols)
         , m_currentArg(-1)
@@ -436,7 +426,7 @@ AssistInterface *InternalCompletionAssistProvider::createAssistInterface(const Q
         int position,
         AssistReason reason) const
 {
-    QTC_ASSERT(textEditorWidget, return 0);
+    QTC_ASSERT(textEditorWidget, return nullptr);
 
     return new CppCompletionAssistInterface(filePath,
                                             textEditorWidget,
@@ -453,9 +443,9 @@ AssistInterface *InternalCompletionAssistProvider::createAssistInterface(const Q
 class CppAssistProposal : public GenericProposal
 {
 public:
-    CppAssistProposal(int cursorPos, GenericProposalModel *model)
+    CppAssistProposal(int cursorPos, GenericProposalModelPtr model)
         : GenericProposal(cursorPos, model)
-        , m_replaceDotForArrow(static_cast<CppAssistProposalModel *>(model)->m_replaceDotForArrow)
+        , m_replaceDotForArrow(model.staticCast<CppAssistProposalModel>()->m_replaceDotForArrow)
     {}
 
     bool isCorrective(TextEditorWidget *) const override { return m_replaceDotForArrow; }
@@ -479,18 +469,16 @@ namespace {
 class ConvertToCompletionItem: protected NameVisitor
 {
     // The completion item.
-    AssistProposalItem *_item;
+    AssistProposalItem *_item = nullptr;
 
     // The current symbol.
-    Symbol *_symbol;
+    Symbol *_symbol = nullptr;
 
     // The pretty printer.
     Overview overview;
 
 public:
     ConvertToCompletionItem()
-        : _item(0)
-        , _symbol(0)
     {
         overview.showReturnTypes = true;
         overview.showArgumentNames = true;
@@ -501,9 +489,9 @@ public:
         //using declaration can be qualified
         if (!symbol || !symbol->name() || (symbol->name()->isQualifiedNameId()
                                            && !symbol->asUsingDeclaration()))
-            return 0;
+            return nullptr;
 
-        AssistProposalItem *previousItem = switchCompletionItem(0);
+        AssistProposalItem *previousItem = switchCompletionItem(nullptr);
         Symbol *previousSymbol = switchSymbol(symbol);
         accept(symbol->unqualifiedName());
         if (_item)
@@ -534,32 +522,32 @@ protected:
         return item;
     }
 
-    void visit(const Identifier *name)
+    void visit(const Identifier *name) override
     {
         _item = newCompletionItem(name);
         if (!_symbol->isScope() || _symbol->isFunction())
             _item->setDetail(overview.prettyType(_symbol->type(), name));
     }
 
-    void visit(const TemplateNameId *name)
+    void visit(const TemplateNameId *name) override
     {
         _item = newCompletionItem(name);
         _item->setText(QString::fromUtf8(name->identifier()->chars(), name->identifier()->size()));
     }
 
-    void visit(const DestructorNameId *name)
+    void visit(const DestructorNameId *name) override
     { _item = newCompletionItem(name); }
 
-    void visit(const OperatorNameId *name)
+    void visit(const OperatorNameId *name) override
     {
         _item = newCompletionItem(name);
         _item->setDetail(overview.prettyType(_symbol->type(), name));
     }
 
-    void visit(const ConversionNameId *name)
+    void visit(const ConversionNameId *name) override
     { _item = newCompletionItem(name); }
 
-    void visit(const QualifiedNameId *name)
+    void visit(const QualifiedNameId *name) override
     { _item = newCompletionItem(name->name()); }
 };
 
@@ -571,7 +559,7 @@ Class *asClassOrTemplateClassType(FullySpecifiedType ty)
         if (Symbol *decl = templ->declaration())
             return decl->asClass();
     }
-    return 0;
+    return nullptr;
 }
 
 Scope *enclosingNonTemplateScope(Symbol *symbol)
@@ -583,7 +571,7 @@ Scope *enclosingNonTemplateScope(Symbol *symbol)
             return scope;
         }
     }
-    return 0;
+    return nullptr;
 }
 
 Function *asFunctionOrTemplateFunctionType(FullySpecifiedType ty)
@@ -594,7 +582,7 @@ Function *asFunctionOrTemplateFunctionType(FullySpecifiedType ty)
         if (Symbol *decl = templ->declaration())
             return decl->asFunction();
     }
-    return 0;
+    return nullptr;
 }
 
 bool isQPrivateSignal(const Symbol *symbol)
@@ -768,7 +756,7 @@ bool canCompleteClassNameAt2ndOr4thConnectArgument(
 ClassOrNamespace *classOrNamespaceFromLookupItem(const LookupItem &lookupItem,
                                                  const LookupContext &context)
 {
-    const Name *name = 0;
+    const Name *name = nullptr;
 
     if (Symbol *d = lookupItem.declaration()) {
         if (Class *k = d->asClass())
@@ -781,29 +769,29 @@ ClassOrNamespace *classOrNamespaceFromLookupItem(const LookupItem &lookupItem,
         if (PointerType *pointerType = type->asPointerType())
             type = pointerType->elementType().simplified();
         else
-            return 0; // not a pointer or a reference to a pointer.
+            return nullptr; // not a pointer or a reference to a pointer.
 
         NamedType *namedType = type->asNamedType();
         if (!namedType) // not a class name.
-            return 0;
+            return nullptr;
 
         name = namedType->name();
     }
 
-    return name ? context.lookupType(name, lookupItem.scope()) : 0;
+    return name ? context.lookupType(name, lookupItem.scope()) : nullptr;
 }
 
 Class *classFromLookupItem(const LookupItem &lookupItem, const LookupContext &context)
 {
     ClassOrNamespace *b = classOrNamespaceFromLookupItem(lookupItem, context);
     if (!b)
-        return 0;
+        return nullptr;
 
     foreach (Symbol *s, b->symbols()) {
         if (Class *klass = s->asClass())
             return klass;
     }
-    return 0;
+    return nullptr;
 }
 
 const Name *minimalName(Symbol *symbol, Scope *targetScope, const LookupContext &context)
@@ -824,15 +812,14 @@ InternalCppCompletionAssistProcessor::InternalCppCompletionAssistProcessor()
 {
 }
 
-InternalCppCompletionAssistProcessor::~InternalCppCompletionAssistProcessor()
-{}
+InternalCppCompletionAssistProcessor::~InternalCppCompletionAssistProcessor() = default;
 
 IAssistProposal * InternalCppCompletionAssistProcessor::perform(const AssistInterface *interface)
 {
     m_interface.reset(static_cast<const CppCompletionAssistInterface *>(interface));
 
     if (interface->reason() != ExplicitlyInvoked && !accepts())
-        return 0;
+        return nullptr;
 
     int index = startCompletionHelper();
     if (index != -1) {
@@ -842,7 +829,7 @@ IAssistProposal * InternalCppCompletionAssistProcessor::perform(const AssistInte
         return createContentProposal();
     }
 
-    return 0;
+    return nullptr;
 }
 
 bool InternalCppCompletionAssistProcessor::accepts() const
@@ -911,13 +898,13 @@ IAssistProposal *InternalCppCompletionAssistProcessor::createContentProposal()
     QSet<QString> processed;
     auto it = m_completions.begin();
     while (it != m_completions.end()) {
-        CppAssistProposalItem *item = static_cast<CppAssistProposalItem *>(*it);
+        auto item = static_cast<CppAssistProposalItem *>(*it);
         if (!processed.contains(item->text()) || item->isSnippet()) {
             ++it;
             if (!item->isSnippet()) {
                 processed.insert(item->text());
                 if (!item->isOverloaded()) {
-                    if (Symbol *symbol = qvariant_cast<Symbol *>(item->data())) {
+                    if (auto symbol = qvariant_cast<Symbol *>(item->data())) {
                         if (Function *funTy = symbol->type()->asFunctionType()) {
                             if (funTy->hasArguments())
                                 item->markAsOverloaded();
@@ -932,16 +919,15 @@ IAssistProposal *InternalCppCompletionAssistProcessor::createContentProposal()
     }
 
     m_model->loadContent(m_completions);
-    return new CppAssistProposal(m_positionForProposal, m_model.take());
+    return new CppAssistProposal(m_positionForProposal, m_model);
 }
 
 IAssistProposal *InternalCppCompletionAssistProcessor::createHintProposal(
     QList<Function *> functionSymbols) const
 {
-    IFunctionHintProposalModel *model =
-            new CppFunctionHintModel(functionSymbols, m_model->m_typeOfExpression);
-    IAssistProposal *proposal = new FunctionHintProposal(m_positionForProposal, model);
-    return proposal;
+    FunctionHintProposalModelPtr model(new CppFunctionHintModel(functionSymbols,
+                                                                m_model->m_typeOfExpression));
+    return new FunctionHintProposal(m_positionForProposal, model);
 }
 
 int InternalCppCompletionAssistProcessor::startOfOperator(int positionInDocument,
@@ -1101,9 +1087,9 @@ int InternalCppCompletionAssistProcessor::startCompletionHelper()
     }
 
     int line = 0, column = 0;
-    Convenience::convertPosition(m_interface->textDocument(), startOfExpression, &line, &column);
+    Utils::Text::convertPosition(m_interface->textDocument(), startOfExpression, &line, &column);
     const QString fileName = m_interface->fileName();
-    return startCompletionInternal(fileName, line, column, expression, endOfExpression);
+    return startCompletionInternal(fileName, line, column - 1, expression, endOfExpression);
 }
 
 bool InternalCppCompletionAssistProcessor::tryObjCCompletion()
@@ -1134,8 +1120,9 @@ bool InternalCppCompletionAssistProcessor::tryObjCCompletion()
     m_model->m_typeOfExpression->init(thisDocument, m_interface->snapshot());
 
     int line = 0, column = 0;
-    Convenience::convertPosition(m_interface->textDocument(), m_interface->position(), &line, &column);
-    Scope *scope = thisDocument->scopeAt(line, column);
+    Utils::Text::convertPosition(m_interface->textDocument(), m_interface->position(), &line,
+                                 &column);
+    Scope *scope = thisDocument->scopeAt(line, column - 1);
     if (!scope)
         return false;
 
@@ -1268,20 +1255,20 @@ bool InternalCppCompletionAssistProcessor::completeInclude(const QTextCursor &cu
     }
 
     // Make completion for all relevant includes
-    ProjectPartHeaderPaths headerPaths = m_interface->headerPaths();
-    const ProjectPartHeaderPath currentFilePath(QFileInfo(m_interface->fileName()).path(),
-                                                ProjectPartHeaderPath::IncludePath);
+    ProjectExplorer::HeaderPaths headerPaths = m_interface->headerPaths();
+    const ProjectExplorer::HeaderPath currentFilePath(QFileInfo(m_interface->fileName()).path(),
+                                                      ProjectExplorer::HeaderPathType::User);
     if (!headerPaths.contains(currentFilePath))
         headerPaths.append(currentFilePath);
 
     const QStringList suffixes = Utils::mimeTypeForName(QLatin1String("text/x-c++hdr")).suffixes();
 
-    foreach (const ProjectPartHeaderPath &headerPath, headerPaths) {
+    foreach (const ProjectExplorer::HeaderPath &headerPath, headerPaths) {
         QString realPath = headerPath.path;
         if (!directoryPrefix.isEmpty()) {
             realPath += QLatin1Char('/');
             realPath += directoryPrefix;
-            if (headerPath.isFrameworkPath())
+            if (headerPath.type == ProjectExplorer::HeaderPathType::Framework)
                 realPath += QLatin1String(".framework/Headers");
         }
         completeInclude(realPath, suffixes);
@@ -1329,7 +1316,8 @@ bool InternalCppCompletionAssistProcessor::objcKeywordsWanted() const
 }
 
 int InternalCppCompletionAssistProcessor::startCompletionInternal(const QString &fileName,
-                                                                  unsigned line, unsigned column,
+                                                                  unsigned line,
+                                                                  unsigned positionInBlock,
                                                                   const QString &expr,
                                                                   int endOfExpression)
 {
@@ -1341,8 +1329,8 @@ int InternalCppCompletionAssistProcessor::startCompletionInternal(const QString
 
     m_model->m_typeOfExpression->init(thisDocument, m_interface->snapshot());
 
-    Scope *scope = thisDocument->scopeAt(line, column);
-    QTC_ASSERT(scope != 0, return -1);
+    Scope *scope = thisDocument->scopeAt(line, positionInBlock);
+    QTC_ASSERT(scope, return -1);
 
     if (expression.isEmpty()) {
         if (m_model->m_completionOperator == T_EOF_SYMBOL || m_model->m_completionOperator == T_COLON_COLON) {
@@ -1470,7 +1458,7 @@ bool InternalCppCompletionAssistProcessor::globalCompletion(Scope *currentScope)
     }
 
     QList<ClassOrNamespace *> usingBindings;
-    ClassOrNamespace *currentBinding = 0;
+    ClassOrNamespace *currentBinding = nullptr;
 
     for (Scope *scope = currentScope; scope; scope = scope->enclosingScope()) {
         if (Block *block = scope->asBlock()) {
@@ -1552,7 +1540,7 @@ bool InternalCppCompletionAssistProcessor::completeMember(const QList<LookupItem
 
     ResolveExpression resolveExpression(context);
 
-    bool *replaceDotForArrow = 0;
+    bool *replaceDotForArrow = nullptr;
     if (!m_interface->languageFeatures().objCEnabled)
         replaceDotForArrow = &m_model->m_replaceDotForArrow;
 
@@ -1749,9 +1737,8 @@ void InternalCppCompletionAssistProcessor::addClassMembersToCompletion(Scope *sc
         else
             addCompletionItem(member);
     }
-    std::set<Class *>::const_iterator citEnd = nestedAnonymouses.end();
-    for (std::set<Class *>::const_iterator cit = nestedAnonymouses.begin(); cit != citEnd; ++cit)
-        addClassMembersToCompletion(*cit, staticLookup);
+    for (Class *klass : nestedAnonymouses)
+        addClassMembersToCompletion(klass, staticLookup);
 }
 
 bool InternalCppCompletionAssistProcessor::completeQtMethod(const QList<LookupItem> &results,
@@ -1847,7 +1834,7 @@ bool InternalCppCompletionAssistProcessor::completeQtMethodClassName(
         return false;
 
     const LookupContext &context = m_model->m_typeOfExpression->context();
-    const QIcon classIcon = Icons::iconForType(Icons::ClassIconType);
+    const QIcon classIcon = Utils::CodeModelIcon::iconForType(Utils::CodeModelIcon::Class);
     Overview overview;
 
     foreach (const LookupItem &lookupItem, results) {
@@ -1912,7 +1899,7 @@ void InternalCppCompletionAssistProcessor::addMacros_helper(const Snapshot &snap
     foreach (const Document::Include &i, doc->resolvedIncludes())
         addMacros_helper(snapshot, i.resolvedFileName(), processed, definedMacros);
 
-    foreach (const Macro &macro, doc->definedMacros()) {
+    foreach (const CPlusPlus::Macro &macro, doc->definedMacros()) {
         const QString macroName = macro.nameToQString();
         if (!macro.isHidden())
             definedMacros->insert(macroName);
@@ -2024,8 +2011,9 @@ bool InternalCppCompletionAssistProcessor::completeConstructorOrFunction(const Q
 
         // get current line and column
         int lineSigned = 0, columnSigned = 0;
-        Convenience::convertPosition(m_interface->textDocument(), m_interface->position(), &lineSigned, &columnSigned);
-        unsigned line = lineSigned, column = columnSigned;
+        Utils::Text::convertPosition(m_interface->textDocument(), m_interface->position(),
+                                     &lineSigned, &columnSigned);
+        unsigned line = lineSigned, column = columnSigned - 1;
 
         // find a scope that encloses the current location, starting from the lastVisibileSymbol
         // and moving outwards
