@@ -1,7 +1,7 @@
 /****************************************************************************
 **
-** Copyright (C) 2014 Digia Plc and/or its subsidiary(-ies).
-** Contact: http://www.qt-project.org/legal
+** Copyright (C) 2016 The Qt Company Ltd.
+** Contact: https://www.qt.io/licensing/
 **
 ** This file is part of Qt Creator.
 **
@@ -9,29 +9,22 @@
 ** Licensees holding valid commercial Qt licenses may use this file in
 ** accordance with the commercial license agreement provided with the
 ** Software or, alternatively, in accordance with the terms contained in
-** a written agreement between you and Digia.  For licensing terms and
-** conditions see http://www.qt.io/licensing.  For further information
-** use the contact form at http://www.qt.io/contact-us.
+** a written agreement between you and The Qt Company. For licensing terms
+** and conditions see https://www.qt.io/terms-conditions. For further
+** information use the contact form at https://www.qt.io/contact-us.
 **
-** GNU Lesser General Public License Usage
-** Alternatively, this file may be used under the terms of the GNU Lesser
-** General Public License version 2.1 or version 3 as published by the Free
-** Software Foundation and appearing in the file LICENSE.LGPLv21 and
-** LICENSE.LGPLv3 included in the packaging of this file.  Please review the
-** following information to ensure the GNU Lesser General Public License
-** requirements will be met: https://www.gnu.org/licenses/lgpl.html and
-** http://www.gnu.org/licenses/old-licenses/lgpl-2.1.html.
-**
-** In addition, as a special exception, Digia gives you certain additional
-** rights.  These rights are described in the Digia Qt LGPL Exception
-** version 1.1, included in the file LGPL_EXCEPTION.txt in this package.
+** GNU General Public License Usage
+** Alternatively, this file may be used under the terms of the GNU
+** General Public License version 3 as published by the Free Software
+** Foundation with exceptions as appearing in the file LICENSE.GPL3-EXCEPT
+** included in the packaging of this file. Please review the following
+** information to ensure the GNU General Public License requirements will
+** be met: https://www.gnu.org/licenses/gpl-3.0.html.
 **
 ****************************************************************************/
 
-#ifndef CPPFINDREFERENCES_H
-#define CPPFINDREFERENCES_H
+#pragma once
 
-#include <cplusplus/DependencyTable.h>
 #include <cplusplus/FindUsages.h>
 
 #include <QMutex>
@@ -47,8 +40,12 @@ class SearchResultItem;
 class SearchResult;
 } // namespace Core
 
+namespace ProjectExplorer {
+class Node;
+}
+
 namespace CppTools {
-class CppModelManagerInterface;
+class CppModelManager;
 
 namespace Internal {
 
@@ -57,6 +54,8 @@ class CppFindReferencesParameters
 public:
     QList<QByteArray> symbolId;
     QByteArray symbolFileName;
+    QString prettySymbolName;
+    QVector<ProjectExplorer::Node *> filesToRename;
 };
 
 class CppFindReferences: public QObject
@@ -64,8 +63,8 @@ class CppFindReferences: public QObject
     Q_OBJECT
 
 public:
-    CppFindReferences(CppModelManagerInterface *modelManager);
-    virtual ~CppFindReferences();
+    explicit CppFindReferences(CppModelManager *modelManager);
+    ~CppFindReferences() override;
 
     QList<int> references(CPlusPlus::Symbol *symbol, const CPlusPlus::LookupContext &context) const;
 
@@ -77,44 +76,25 @@ public:
     void findMacroUses(const CPlusPlus::Macro &macro);
     void renameMacroUses(const CPlusPlus::Macro &macro, const QString &replacement = QString());
 
-    CPlusPlus::DependencyTable updateDependencyTable(CPlusPlus::Snapshot snapshot);
-
-public slots:
-    void flushDependencyTable();
-
-private slots:
-    void displayResults(int first, int last);
-    void searchFinished();
-    void cancel();
-    void setPaused(bool paused);
-    void openEditor(const Core::SearchResultItem &item);
+private:
     void onReplaceButtonClicked(const QString &text, const QList<Core::SearchResultItem> &items, bool preserveCase);
     void searchAgain();
 
-private:
     void findUsages(CPlusPlus::Symbol *symbol, const CPlusPlus::LookupContext &context,
                     const QString &replacement, bool replace);
     void findMacroUses(const CPlusPlus::Macro &macro, const QString &replacement,
                        bool replace);
     void findAll_helper(Core::SearchResult *search, CPlusPlus::Symbol *symbol,
                         const CPlusPlus::LookupContext &context);
-    CPlusPlus::DependencyTable dependencyTable() const;
-    void setDependencyTable(const CPlusPlus::DependencyTable &newTable);
     void createWatcher(const QFuture<CPlusPlus::Usage> &future, Core::SearchResult *search);
     CPlusPlus::Symbol *findSymbol(const CppFindReferencesParameters &parameters,
                     const CPlusPlus::Snapshot &snapshot, CPlusPlus::LookupContext *context);
 
 private:
-    QPointer<CppModelManagerInterface> m_modelManager;
-    QMap<QFutureWatcher<CPlusPlus::Usage> *, QPointer<Core::SearchResult> > m_watchers;
-
-    mutable QMutex m_depsLock;
-    CPlusPlus::DependencyTable m_deps;
+    QPointer<CppModelManager> m_modelManager;
 };
 
 } // namespace Internal
 } // namespace CppTools
 
 Q_DECLARE_METATYPE(CppTools::Internal::CppFindReferencesParameters)
-
-#endif // CPPFINDREFERENCES_H
