@@ -95,22 +95,22 @@ public:
         StringTable::destroy();
         delete m_cppFileSettingsPage;
         delete m_cppCodeModelSettingsPage;
-        delete m_cppCodeStyleSettingsPage;
+        if (m_cppCodeStyleSettingsPage)
+            delete m_cppCodeStyleSettingsPage;
     }
 
     QSharedPointer<CppCodeModelSettings> m_codeModelSettings;
     CppToolsSettings *m_settings = nullptr;
     CppFileSettingsPage *m_cppFileSettingsPage = nullptr;
     CppCodeModelSettingsPage *m_cppCodeModelSettingsPage = nullptr;
-    CppCodeStyleSettingsPage *m_cppCodeStyleSettingsPage = nullptr;
+    QPointer<CppCodeStyleSettingsPage> m_cppCodeStyleSettingsPage = nullptr;
 };
 
 CppToolsPlugin::CppToolsPlugin()
     : m_fileSettings(new CppFileSettings)
 {
     m_instance = this;
-    auto bridgeImplementation = std::unique_ptr<CppToolsBridgeQtCreatorImplementation>(new CppToolsBridgeQtCreatorImplementation);
-    CppToolsBridge::setCppToolsBridgeImplementation(std::move(bridgeImplementation));
+    CppToolsBridge::setCppToolsBridgeImplementation(std::make_unique<CppToolsBridgeQtCreatorImplementation>());
 }
 
 CppToolsPlugin::~CppToolsPlugin()
@@ -140,6 +140,11 @@ QString CppToolsPlugin::licenseTemplate()
     return m_instance->m_fileSettings->licenseTemplate();
 }
 
+bool CppToolsPlugin::usePragmaOnce()
+{
+    return m_instance->m_fileSettings->headerPragmaOnce;
+}
+
 const QStringList &CppToolsPlugin::headerSearchPaths()
 {
     return m_instance->m_fileSettings->headerSearchPaths;
@@ -204,6 +209,11 @@ bool CppToolsPlugin::initialize(const QStringList &arguments, QString *error)
                                     tr("The configured path to the license template"),
                                     []() { return CppToolsPlugin::licenseTemplatePath().toString(); });
 
+    expander->registerVariable(
+                "Cpp:PragmaOnce",
+                tr("Insert \"#pragma once\" instead of \"#ifndef\" include guards into header file"),
+                [] { return usePragmaOnce() ? QString("true") : QString(); });
+
     return true;
 }
 
