@@ -74,7 +74,7 @@ bool FindMacroHandler::executeEvent(const MacroEvent &macroEvent)
     if (!aggregate)
         return false;
 
-    Core::IFindSupport *currentFind = aggregate->component<Core::IFindSupport>();
+    auto currentFind = aggregate->component<Core::IFindSupport>();
     if (!currentFind)
         return false;
 
@@ -91,6 +91,7 @@ bool FindMacroHandler::executeEvent(const MacroEvent &macroEvent)
         currentFind->replace(macroEvent.value(BEFORE).toString(),
                              macroEvent.value(AFTER).toString(),
                              (Core::FindFlags)macroEvent.value(FLAGS).toInt());
+        break;
     case REPLACESTEP:
         currentFind->replaceStep(macroEvent.value(BEFORE).toString(),
                               macroEvent.value(AFTER).toString(),
@@ -189,9 +190,8 @@ void FindMacroHandler::changeEditor(Core::IEditor *editor)
 
     Aggregation::Aggregate *aggregate = Aggregation::Aggregate::parentAggregate(editor->widget());
     if (aggregate) {
-        Core::IFindSupport *currentFind = aggregate->component<Core::IFindSupport>();
-        if (currentFind) {
-            MacroTextFind *macroFind = qobject_cast<MacroTextFind *>(currentFind);
+        if (auto currentFind = aggregate->component<Core::IFindSupport>()) {
+            auto macroFind = qobject_cast<MacroTextFind *>(currentFind);
             if (macroFind)
                 return;
 
