@@ -28,10 +28,12 @@
 #include "buildprogress.h"
 #include "buildsteplist.h"
 #include "compileoutputwindow.h"
+#include "kit.h"
 #include "project.h"
 #include "projectexplorer.h"
 #include "projectexplorersettings.h"
 #include "target.h"
+#include "task.h"
 #include "taskwindow.h"
 #include "taskhub.h"
 
@@ -59,73 +61,51 @@ namespace ProjectExplorer {
 
 static QString msgProgress(int progress, int total)
 {
-    return BuildManager::tr("Finished %1 of %n steps", 0, total).arg(progress);
+    return BuildManager::tr("Finished %1 of %n steps", nullptr, total).arg(progress);
 }
 
-struct BuildManagerPrivate
+class BuildManagerPrivate
 {
-    BuildManagerPrivate();
-
-    Internal::CompileOutputWindow *m_outputWindow;
-    TaskHub *m_taskHub;
-    Internal::TaskWindow *m_taskWindow;
+public:
+    Internal::CompileOutputWindow *m_outputWindow = nullptr;
+    Internal::TaskWindow *m_taskWindow = nullptr;
 
     QList<BuildStep *> m_buildQueue;
     QList<bool> m_enabledState;
     QStringList m_stepNames;
-    bool m_running;
-    QFutureWatcher<bool> m_watcher;
-    QFutureInterface<bool> m_futureInterfaceForAysnc;
-    BuildStep *m_currentBuildStep;
+    int m_progress = 0;
+    int m_maxProgress = 0;
+    bool m_running = false;
+    // is set to true while canceling, so that nextBuildStep knows that the BuildStep finished because of canceling
+    bool m_skipDisabled = false;
+    bool m_canceling = false;
+    bool m_lastStepSucceeded = true;
+    BuildStep *m_currentBuildStep = nullptr;
     QString m_currentConfiguration;
     // used to decide if we are building a project to decide when to emit buildStateChanged(Project *)
     QHash<Project *, int>  m_activeBuildSteps;
     QHash<Target *, int> m_activeBuildStepsPerTarget;
     QHash<ProjectConfiguration *, int> m_activeBuildStepsPerProjectConfiguration;
-    Project *m_previousBuildStepProject;
-    // is set to true while canceling, so that nextBuildStep knows that the BuildStep finished because of canceling
-    bool m_skipDisabled;
-    bool m_canceling;
+    Project *m_previousBuildStepProject = nullptr;
 
     // Progress reporting to the progress manager
-    int m_progress;
-    int m_maxProgress;
-    QFutureInterface<void> *m_progressFutureInterface;
+    QFutureInterface<void> *m_progressFutureInterface = nullptr;
     QFutureWatcher<void> m_progressWatcher;
     QPointer<FutureProgress> m_futureProgress;
 
     QElapsedTimer m_elapsed;
 };
 
-BuildManagerPrivate::BuildManagerPrivate() :
-    m_running(false)
-  , m_previousBuildStepProject(0)
-  , m_skipDisabled(false)
-  , m_canceling(false)
-  , m_maxProgress(0)
-  , m_progressFutureInterface(0)
-{
-}
-
-static BuildManagerPrivate *d = 0;
-static BuildManager *m_instance = 0;
+static BuildManagerPrivate *d = nullptr;
+static BuildManager *m_instance = nullptr;
 
 BuildManager::BuildManager(QObject *parent, QAction *cancelBuildAction)
     : QObject(parent)
 {
+    QTC_CHECK(!m_instance);
     m_instance = this;
     d = new BuildManagerPrivate;
 
-    connect(&d->m_watcher, &QFutureWatcherBase::finished,
-            this, &BuildManager::nextBuildQueue, Qt::QueuedConnection);
-
-    connect(&d->m_watcher, &QFutureWatcherBase::progressValueChanged,
-            this, &BuildManager::progressChanged);
-    connect(&d->m_watcher, &QFutureWatcherBase::progressTextChanged,
-            this, &BuildManager::progressTextChanged);
-    connect(&d->m_watcher, &QFutureWatcherBase::progressRangeChanged,
-            this, &BuildManager::progressChanged);
-
     connect(SessionManager::instance(), &SessionManager::aboutToRemoveProject,
             this, &BuildManager::aboutToRemoveProject);
 
@@ -168,7 +148,7 @@ void BuildManager::extensionsInitialized()
 BuildManager::~BuildManager()
 {
     cancel();
-    m_instance = 0;
+    m_instance = nullptr;
     ExtensionSystem::PluginManager::removeObject(d->m_taskWindow);
     delete d->m_taskWindow;
 
@@ -176,6 +156,7 @@ BuildManager::~BuildManager()
     delete d->m_outputWindow;
 
     delete d;
+    d = nullptr;
 }
 
 void BuildManager::aboutToRemoveProject(Project *p)
@@ -211,14 +192,9 @@ void BuildManager::cancel()
         if (d->m_canceling)
             return;
         d->m_canceling = true;
-        d->m_watcher.cancel();
-        if (d->m_currentBuildStep->runInGuiThread()) {
-            d->m_currentBuildStep->cancel();
-            while (d->m_canceling)
-                QApplication::processEvents(QEventLoop::ExcludeUserInputEvents);
-        } else {
-            d->m_watcher.waitForFinished();
-        }
+        d->m_currentBuildStep->cancel();
+        while (d->m_canceling)
+            QApplication::processEvents(QEventLoop::ExcludeUserInputEvents); // TODO: Is this really necessary?
     }
 }
 
@@ -235,14 +211,14 @@ void BuildManager::finish()
     QString time = format.toString(QLatin1String("h:mm:ss"));
     if (time.startsWith(QLatin1String("0:")))
         time.remove(0, 2); // Don't display zero hours
-    m_instance->addToOutputWindow(tr("Elapsed time: %1.") .arg(time), BuildStep::MessageOutput);
+    m_instance->addToOutputWindow(tr("Elapsed time: %1.") .arg(time), BuildStep::OutputFormat::NormalMessage);
 
     QApplication::alert(ICore::mainWindow(), 3000);
 }
 
 void BuildManager::emitCancelMessage()
 {
-    m_instance->addToOutputWindow(tr("Canceled build/deployment."), BuildStep::ErrorMessageOutput);
+    m_instance->addToOutputWindow(tr("Canceled build/deployment."), BuildStep::OutputFormat::ErrorMessage);
 }
 
 void BuildManager::clearBuildQueue()
@@ -256,15 +232,15 @@ void BuildManager::clearBuildQueue()
     d->m_buildQueue.clear();
     d->m_enabledState.clear();
     d->m_running = false;
-    d->m_previousBuildStepProject = 0;
-    d->m_currentBuildStep = 0;
+    d->m_previousBuildStepProject = nullptr;
+    d->m_currentBuildStep = nullptr;
 
     d->m_progressFutureInterface->reportCanceled();
     d->m_progressFutureInterface->reportFinished();
     d->m_progressWatcher.setFuture(QFuture<void>());
     delete d->m_progressFutureInterface;
-    d->m_progressFutureInterface = 0;
-    d->m_futureProgress = 0;
+    d->m_progressFutureInterface = nullptr;
+    d->m_futureProgress = nullptr;
     d->m_maxProgress = 0;
 
     emit m_instance->buildQueueFinished(false);
@@ -348,7 +324,7 @@ void BuildManager::addToOutputWindow(const QString &string, BuildStep::OutputFor
                                      BuildStep::OutputNewlineSetting newlineSettings)
 {
     QString stringToWrite;
-    if (format == BuildStep::MessageOutput || format == BuildStep::ErrorMessageOutput) {
+    if (format == BuildStep::OutputFormat::NormalMessage || format == BuildStep::OutputFormat::ErrorMessage) {
         stringToWrite = QTime::currentTime().toString();
         stringToWrite += QLatin1String(": ");
     }
@@ -358,14 +334,6 @@ void BuildManager::addToOutputWindow(const QString &string, BuildStep::OutputFor
     d->m_outputWindow->appendText(stringToWrite, format);
 }
 
-void BuildManager::buildStepFinishedAsync()
-{
-    disconnect(d->m_currentBuildStep, &BuildStep::finished,
-               this, &BuildManager::buildStepFinishedAsync);
-    d->m_futureInterfaceForAysnc = QFutureInterface<bool>();
-    nextBuildQueue();
-}
-
 void BuildManager::nextBuildQueue()
 {
     d->m_outputWindow->flush();
@@ -389,45 +357,32 @@ void BuildManager::nextBuildQueue()
     d->m_progressFutureInterface->setProgressValueAndText(d->m_progress*100, msgProgress(d->m_progress, d->m_maxProgress));
     decrementActiveBuildSteps(d->m_currentBuildStep);
 
-    bool result = d->m_skipDisabled || d->m_watcher.result();
-    if (!result) {
+    const bool success = d->m_skipDisabled || d->m_lastStepSucceeded;
+    if (success) {
+        nextStep();
+    } else {
         // Build Failure
+        Target *t = d->m_currentBuildStep->target();
         const QString projectName = d->m_currentBuildStep->project()->displayName();
-        const QString targetName = d->m_currentBuildStep->target()->displayName();
-        addToOutputWindow(tr("Error while building/deploying project %1 (kit: %2)").arg(projectName, targetName), BuildStep::ErrorOutput);
-        addToOutputWindow(tr("When executing step \"%1\"").arg(d->m_currentBuildStep->displayName()), BuildStep::ErrorOutput);
+        const QString targetName = t->displayName();
+        addToOutputWindow(tr("Error while building/deploying project %1 (kit: %2)").arg(projectName, targetName), BuildStep::OutputFormat::Stderr);
+        const QList<Task> kitTasks = t->kit()->validate();
+        if (!kitTasks.isEmpty()) {
+            addToOutputWindow(tr("The kit %1 has configuration issues which might be the root cause for this problem.")
+                              .arg(targetName), BuildStep::OutputFormat::Stderr);
+        }
+        addToOutputWindow(tr("When executing step \"%1\"").arg(d->m_currentBuildStep->displayName()), BuildStep::OutputFormat::Stderr);
         // NBS TODO fix in qtconcurrent
         d->m_progressFutureInterface->setProgressValueAndText(d->m_progress*100, tr("Error while building/deploying project %1 (kit: %2)").arg(projectName, targetName));
-    }
 
-    if (result)
-        nextStep();
-    else
         clearBuildQueue();
-}
-
-void BuildManager::progressChanged()
-{
-    if (!d->m_progressFutureInterface)
-        return;
-    int range = d->m_watcher.progressMaximum() - d->m_watcher.progressMinimum();
-    if (range != 0) {
-        int percent = (d->m_watcher.progressValue() - d->m_watcher.progressMinimum()) * 100 / range;
-        d->m_progressFutureInterface->setProgressValueAndText(d->m_progress * 100 + percent, msgProgress(d->m_progress, d->m_maxProgress)
-                                                              + QLatin1Char('\n') + d->m_watcher.progressText());
     }
 }
 
-void BuildManager::progressTextChanged()
+void BuildManager::progressChanged(int percent, const QString &text)
 {
-    if (!d->m_progressFutureInterface)
-        return;
-    int range = d->m_watcher.progressMaximum() - d->m_watcher.progressMinimum();
-    int percent = 0;
-    if (range != 0)
-        percent = (d->m_watcher.progressValue() - d->m_watcher.progressMinimum()) * 100 / range;
-    d->m_progressFutureInterface->setProgressValueAndText(d->m_progress*100 + percent, msgProgress(d->m_progress, d->m_maxProgress) +
-                                                          QLatin1Char('\n') + d->m_watcher.progressText());
+    if (d->m_progressFutureInterface)
+        d->m_progressFutureInterface->setProgressValueAndText(percent, text);
 }
 
 void BuildManager::nextStep()
@@ -443,39 +398,41 @@ void BuildManager::nextStep()
         if (d->m_currentBuildStep->project() != d->m_previousBuildStepProject) {
             const QString projectName = d->m_currentBuildStep->project()->displayName();
             addToOutputWindow(tr("Running steps for project %1...")
-                              .arg(projectName), BuildStep::MessageOutput);
+                              .arg(projectName), BuildStep::OutputFormat::NormalMessage);
             d->m_previousBuildStepProject = d->m_currentBuildStep->project();
         }
 
         if (d->m_skipDisabled) {
             addToOutputWindow(tr("Skipping disabled step %1.")
-                              .arg(d->m_currentBuildStep->displayName()), BuildStep::MessageOutput);
+                              .arg(d->m_currentBuildStep->displayName()), BuildStep::OutputFormat::NormalMessage);
             nextBuildQueue();
             return;
         }
 
-        if (d->m_currentBuildStep->runInGuiThread()) {
-            connect(d->m_currentBuildStep, &BuildStep::finished,
-                    m_instance, &BuildManager::buildStepFinishedAsync);
-            d->m_watcher.setFuture(d->m_futureInterfaceForAysnc.future());
-            d->m_currentBuildStep->run(d->m_futureInterfaceForAysnc);
-        } else {
-            d->m_watcher.setFuture(Utils::runAsync(&BuildStep::run, d->m_currentBuildStep));
-        }
+        static const auto finishedHandler = [](bool success)  {
+            d->m_lastStepSucceeded = success;
+            disconnect(d->m_currentBuildStep, nullptr, instance(), nullptr);
+            BuildManager::nextBuildQueue();
+        };
+        connect(d->m_currentBuildStep, &BuildStep::finished, instance(), finishedHandler,
+                Qt::QueuedConnection);
+        connect(d->m_currentBuildStep, &BuildStep::progress,
+                instance(), &BuildManager::progressChanged);
+        d->m_currentBuildStep->run();
     } else {
         d->m_running = false;
-        d->m_previousBuildStepProject = 0;
+        d->m_previousBuildStepProject = nullptr;
         d->m_progressFutureInterface->reportFinished();
         d->m_progressWatcher.setFuture(QFuture<void>());
-        d->m_currentBuildStep = 0;
+        d->m_currentBuildStep = nullptr;
         delete d->m_progressFutureInterface;
-        d->m_progressFutureInterface = 0;
+        d->m_progressFutureInterface = nullptr;
         d->m_maxProgress = 0;
         emit m_instance->buildQueueFinished(true);
     }
 }
 
-bool BuildManager::buildQueueAppend(QList<BuildStep *> steps, QStringList names, const QStringList &preambleMessage)
+bool BuildManager::buildQueueAppend(const QList<BuildStep *> &steps, QStringList names, const QStringList &preambleMessage)
 {
     if (!d->m_running) {
         d->m_outputWindow->clearContents();
@@ -484,10 +441,9 @@ bool BuildManager::buildQueueAppend(QList<BuildStep *> steps, QStringList names,
         TaskHub::clearTasks(Constants::TASK_CATEGORY_DEPLOYMENT);
 
         foreach (const QString &str, preambleMessage)
-            addToOutputWindow(str, BuildStep::MessageOutput, BuildStep::DontAppendNewline);
+            addToOutputWindow(str, BuildStep::OutputFormat::NormalMessage, BuildStep::DontAppendNewline);
     }
 
-    QList<const BuildStep *> earlierSteps;
     int count = steps.size();
     bool init = true;
     int i = 0;
@@ -496,10 +452,9 @@ bool BuildManager::buildQueueAppend(QList<BuildStep *> steps, QStringList names,
         connect(bs, &BuildStep::addTask, m_instance, &BuildManager::addToTaskWindow);
         connect(bs, &BuildStep::addOutput, m_instance, &BuildManager::addToOutputWindow);
         if (bs->enabled()) {
-            init = bs->init(earlierSteps);
+            init = bs->init();
             if (!init)
                 break;
-            earlierSteps.append(bs);
         }
     }
     if (!init) {
@@ -509,8 +464,8 @@ bool BuildManager::buildQueueAppend(QList<BuildStep *> steps, QStringList names,
         // print something for the user
         const QString projectName = bs->project()->displayName();
         const QString targetName = bs->target()->displayName();
-        addToOutputWindow(tr("Error while building/deploying project %1 (kit: %2)").arg(projectName, targetName), BuildStep::ErrorOutput);
-        addToOutputWindow(tr("When executing step \"%1\"").arg(bs->displayName()), BuildStep::ErrorOutput);
+        addToOutputWindow(tr("Error while building/deploying project %1 (kit: %2)").arg(projectName, targetName), BuildStep::OutputFormat::Stderr);
+        addToOutputWindow(tr("When executing step \"%1\"").arg(bs->displayName()), BuildStep::OutputFormat::Stderr);
 
         // disconnect the buildsteps again
         for (int j = 0; j <= i; ++j)
@@ -531,23 +486,25 @@ bool BuildManager::buildQueueAppend(QList<BuildStep *> steps, QStringList names,
     return true;
 }
 
-bool BuildManager::buildList(BuildStepList *bsl, const QString &stepListName)
+bool BuildManager::buildList(BuildStepList *bsl)
 {
-    return buildLists(QList<BuildStepList *>() << bsl, QStringList() << stepListName);
+    return buildLists({bsl});
 }
 
-bool BuildManager::buildLists(QList<BuildStepList *> bsls, const QStringList &stepListNames, const QStringList &preambelMessage)
+bool BuildManager::buildLists(QList<BuildStepList *> bsls, const QStringList &preambelMessage)
 {
     QList<BuildStep *> steps;
-    foreach (BuildStepList *list, bsls)
+    QStringList stepListNames;
+    foreach (BuildStepList *list, bsls) {
         steps.append(list->steps());
+        stepListNames.append(ProjectExplorerPlugin::displayNameForStepId(list->id()));
+    }
 
     QStringList names;
     names.reserve(steps.size());
     for (int i = 0; i < bsls.size(); ++i) {
-        for (int j = 0; j < bsls.at(i)->steps().size(); ++j) {
+        for (int j = 0; j < bsls.at(i)->count(); ++j)
             names.append(stepListNames.at(i));
-        }
     }
 
     bool success = buildQueueAppend(steps, names, preambelMessage);
@@ -564,7 +521,7 @@ bool BuildManager::buildLists(QList<BuildStepList *> bsls, const QStringList &st
 
 void BuildManager::appendStep(BuildStep *step, const QString &name)
 {
-    bool success = buildQueueAppend(QList<BuildStep *>() << step, QStringList() << name);
+    bool success = buildQueueAppend({step}, {name});
     if (!success) {
         d->m_outputWindow->popup(IOutputPane::NoModeSwitch);
         return;
@@ -575,26 +532,26 @@ void BuildManager::appendStep(BuildStep *step, const QString &name)
 }
 
 template <class T>
-int count(const QHash<T *, int> &hash, T *key)
+int count(const QHash<T *, int> &hash, const T *key)
 {
-    typename QHash<T *, int>::const_iterator it = hash.find(key);
+    typename QHash<T *, int>::const_iterator it = hash.find(const_cast<T *>(key));
     typename QHash<T *, int>::const_iterator end = hash.end();
     if (it != end)
         return *it;
     return 0;
 }
 
-bool BuildManager::isBuilding(Project *pro)
+bool BuildManager::isBuilding(const Project *pro)
 {
     return count(d->m_activeBuildSteps, pro) > 0;
 }
 
-bool BuildManager::isBuilding(Target *t)
+bool BuildManager::isBuilding(const Target *t)
 {
     return count(d->m_activeBuildStepsPerTarget, t) > 0;
 }
 
-bool BuildManager::isBuilding(ProjectConfiguration *p)
+bool BuildManager::isBuilding(const ProjectConfiguration *p)
 {
     return count(d->m_activeBuildStepsPerProjectConfiguration, p) > 0;
 }
@@ -653,8 +610,8 @@ void BuildManager::decrementActiveBuildSteps(BuildStep *bs)
 
 void BuildManager::disconnectOutput(BuildStep *bs)
 {
-    disconnect(bs, &BuildStep::addTask, m_instance, 0);
-    disconnect(bs, &BuildStep::addOutput, m_instance, 0);
+    disconnect(bs, &BuildStep::addTask, m_instance, nullptr);
+    disconnect(bs, &BuildStep::addOutput, m_instance, nullptr);
 }
 
 } // namespace ProjectExplorer
