@@ -31,12 +31,15 @@
 #include "buildtargetinfo.h"
 #include "devicesupport/idevice.h"
 
+#include <utils/environment.h>
 #include <utils/port.h>
 #include <utils/processhandle.h>
 #include <utils/qtcassert.h>
 #include <utils/icon.h>
 
+#include <QHash>
 #include <QPointer>
+#include <QVariant>
 #include <QWidget>
 
 #include <functional>
@@ -45,13 +48,14 @@
 namespace Utils { class OutputFormatter; }
 
 namespace ProjectExplorer {
-class Abi;
 class BuildConfiguration;
-class IRunConfigurationAspect;
-class IRunConfigurationFactory;
+class GlobalOrProjectAspect;
+class Node;
+class RunConfigurationFactory;
 class RunConfiguration;
-class RunConfigWidget;
+class RunConfigurationCreationInfo;
 class RunControl;
+class RunWorkerFactory;
 class Target;
 
 namespace Internal {
@@ -71,22 +75,21 @@ class PROJECTEXPLORER_EXPORT ISettingsAspect : public QObject
     Q_OBJECT
 
 public:
-    ISettingsAspect() { }
-
     /// Create a configuration widget for this settings aspect.
-    virtual QWidget *createConfigWidget(QWidget *parent) = 0;
-    /// "Virtual default constructor"
-    virtual ISettingsAspect *create() const = 0;
-    /// "Virtual copy constructor"
-    ISettingsAspect *clone() const;
+    using ConfigWidgetCreator = std::function<QWidget *()>;
+
+    explicit ISettingsAspect(const ConfigWidgetCreator &configWidgetCreator);
+    QWidget *createConfigWidget() const;
 
 protected:
     ///
-    friend class IRunConfigurationAspect;
+    friend class GlobalOrProjectAspect;
     /// Converts current object into map for storage.
     virtual void toMap(QVariantMap &map) const = 0;
     /// Read object state from @p map.
     virtual void fromMap(const QVariantMap &map) = 0;
+
+    ConfigWidgetCreator m_configWidgetCreator;
 };
 
 
@@ -96,28 +99,17 @@ protected:
  *
  */
 
-class PROJECTEXPLORER_EXPORT IRunConfigurationAspect : public QObject
+class PROJECTEXPLORER_EXPORT GlobalOrProjectAspect : public ProjectConfigurationAspect
 {
     Q_OBJECT
 
 public:
-    explicit IRunConfigurationAspect(RunConfiguration *runConfig);
-    ~IRunConfigurationAspect() override;
+    GlobalOrProjectAspect();
+    ~GlobalOrProjectAspect() override;
 
-    using RunConfigWidgetCreator = std::function<RunConfigWidget *()>;
-    void setRunConfigWidgetCreator(const RunConfigWidgetCreator &runConfigWidgetCreator);
-    RunConfigWidget *createConfigurationWidget() const;
-    void copyFrom(IRunConfigurationAspect *other);
-
-    void setId(Core::Id id) { m_id = id; }
-    void setDisplayName(const QString &displayName) { m_displayName = displayName; }
-    void setSettingsKey(const QString &settingsKey) { m_settingsKey = settingsKey; }
     void setProjectSettings(ISettingsAspect *settings);
     void setGlobalSettings(ISettingsAspect *settings);
 
-    Core::Id id() const { return m_id; }
-    QString displayName() const { return m_displayName; }
-    QString settingsKey() const { return  m_settingsKey; }
     bool isUsingGlobalSettings() const { return m_useGlobalSettings; }
     void setUsingGlobalSettings(bool value);
     void resetProjectToGlobalSettings();
@@ -125,80 +117,32 @@ public:
     ISettingsAspect *projectSettings() const { return m_projectSettings; }
     ISettingsAspect *globalSettings() const { return m_globalSettings; }
     ISettingsAspect *currentSettings() const;
-    RunConfiguration *runConfiguration() const { return m_runConfiguration; }
 
 protected:
     friend class RunConfiguration;
-    virtual void fromMap(const QVariantMap &map);
-    virtual void toMap(QVariantMap &data) const;
+    void fromMap(const QVariantMap &map) override;
+    void toMap(QVariantMap &data) const override;
 
 private:
-    Core::Id m_id;
-    QString m_displayName;
-    QString m_settingsKey; // Name of data in settings.
     bool m_useGlobalSettings = false;
-    RunConfiguration *m_runConfiguration = nullptr;
     ISettingsAspect *m_projectSettings = nullptr; // Owned if present.
     ISettingsAspect *m_globalSettings = nullptr;  // Not owned.
-    RunConfigWidgetCreator m_runConfigWidgetCreator;
 };
 
 class PROJECTEXPLORER_EXPORT Runnable
 {
-    struct Concept
-    {
-        virtual ~Concept() {}
-        virtual Concept *clone() const = 0;
-        virtual bool canReUseOutputPane(const std::unique_ptr<Concept> &other) const = 0;
-        virtual QString displayName() const = 0;
-        virtual void *typeId() const = 0;
-    };
-
-    template <class T>
-    struct Model : public Concept
-    {
-        Model(const T &data) : m_data(data) {}
-
-        Concept *clone() const override { return new Model(*this); }
-
-        bool canReUseOutputPane(const std::unique_ptr<Concept> &other) const override
-        {
-            if (!other.get())
-                return false;
-            if (other->typeId() != typeId())
-                return false;
-            auto that = static_cast<const Model<T> *>(other.get());
-            return m_data == that->m_data;
-        }
-
-        QString displayName() const override { return m_data.displayName(); }
-
-        void *typeId() const override { return T::staticTypeId; }
-
-        T m_data;
-    };
-
 public:
     Runnable() = default;
-    Runnable(const Runnable &other) : d(other.d ? other.d->clone() : nullptr) { }
-    Runnable(Runnable &&other) : d(std::move(other.d)) {}
-    template <class T> Runnable(const T &data) : d(new Model<T>(data)) {}
 
-    void operator=(Runnable other) { d = std::move(other.d); }
+    QString executable;
+    QString commandLineArguments;
+    QString workingDirectory;
+    Utils::Environment environment;
+    IDevice::ConstPtr device; // Override the kit's device. Keep unset by default.
+    QHash<Core::Id, QVariant> extraData;
 
-    template <class T> bool is() const {
-        return d.get() && (d.get()->typeId() == T::staticTypeId);
-    }
-
-    template <class T> const T &as() const {
-        return static_cast<Model<T> *>(d.get())->m_data;
-    }
-
-    bool canReUseOutputPane(const Runnable &other) const;
-    QString displayName() const { return d ? d->displayName() : QString(); }
-
-private:
-    std::unique_ptr<Concept> d;
+    // FIXME: Not necessarily a display name
+    QString displayName() const { return executable; }
 };
 
 // Documentation inside.
@@ -213,7 +157,7 @@ public:
 
     QString disabledReason() const override;
 
-    virtual QWidget *createConfigurationWidget() = 0;
+    virtual QWidget *createConfigurationWidget();
 
     virtual bool isConfigured() const;
     // Pop up configuration dialog in case for example the executable is missing.
@@ -224,37 +168,33 @@ public:
     Target *target() const;
     Project *project() const override;
 
-    virtual Utils::OutputFormatter *createOutputFormatter() const;
+    Utils::OutputFormatter *createOutputFormatter() const;
 
     bool fromMap(const QVariantMap &map) override;
     QVariantMap toMap() const override;
 
-    QList<IRunConfigurationAspect *> extraAspects() const;
-    IRunConfigurationAspect *extraAspect(Core::Id id) const;
+    virtual Runnable runnable() const;
 
-    template <typename T> T *extraAspect() const
+    // Return a handle to the build system target that created this run configuration.
+    // May return an empty string if no target built the executable!
+    QString buildKey() const { return m_buildKey; }
+    // The BuildTargetInfo corresponding to the buildKey.
+    BuildTargetInfo buildTargetInfo() const;
+
+    static RunConfiguration *startupRunConfiguration();
+    virtual bool canRunForNode(const ProjectExplorer::Node *) const { return false; }
+
+    template <class T = ISettingsAspect> T *currentSettings(Core::Id id) const
     {
-        foreach (IRunConfigurationAspect *aspect, m_aspects)
-            if (T *result = qobject_cast<T *>(aspect))
-                return result;
+        if (auto a = qobject_cast<GlobalOrProjectAspect *>(aspect(id)))
+            return qobject_cast<T *>(a->currentSettings());
         return nullptr;
     }
 
-    virtual Runnable runnable() const;
-    virtual Abi abi() const;
-
-    // Return the name of the build system target that created this run configuration.
-    // May return an empty string if no target built the executable!
-    virtual QString buildSystemTarget() const { return QString(); }
-
-    void addExtraAspect(IRunConfigurationAspect *aspect);
-
-    static RunConfiguration *startupRunConfiguration();
-
-    using AspectFactory = std::function<IRunConfigurationAspect *(RunConfiguration *)>;
+    using AspectFactory = std::function<ProjectConfigurationAspect *(Target *)>;
     template <class T> static void registerAspect()
     {
-        addAspectFactory([](RunConfiguration *rc) { return new T(rc); });
+        addAspectFactory([](Target *target) { return new T(target); });
     }
 
 signals:
@@ -267,93 +207,108 @@ protected:
     /// convenience function to get current build configuration.
     BuildConfiguration *activeBuildConfiguration() const;
 
+    template<class T> void setOutputFormatter()
+    {
+        m_outputFormatterCreator = [](Project *project) { return new T(project); };
+    }
+
     virtual void updateEnabledState();
+    virtual void doAdditionalSetup(const RunConfigurationCreationInfo &) {}
 
 private:
     static void addAspectFactory(const AspectFactory &aspectFactory);
 
-    friend class IRunConfigurationFactory;
+    friend class RunConfigurationCreationInfo;
 
-    QList<IRunConfigurationAspect *> m_aspects;
+    QString m_buildKey;
+    std::function<Utils::OutputFormatter *(Project *)> m_outputFormatterCreator;
 };
 
 class RunConfigurationCreationInfo
 {
 public:
-    RunConfigurationCreationInfo(const IRunConfigurationFactory *factory, Core::Id id,
-                                 QString extra, QString displayName)
-        : factory(factory) , id(id) , extra(extra) , displayName(displayName) {}
+    enum CreationMode {AlwaysCreate, ManualCreationOnly};
+    RunConfiguration *create(Target *target) const;
 
-    const IRunConfigurationFactory *factory = nullptr;
+    const RunConfigurationFactory *factory = nullptr;
     Core::Id id;
-    QString extra;
+    QString buildKey;
     QString displayName;
+    QString displayNameUniquifier;
+    CreationMode creationMode = AlwaysCreate;
+    bool useTerminal = false;
 };
 
-class PROJECTEXPLORER_EXPORT IRunConfigurationFactory : public QObject
+class PROJECTEXPLORER_EXPORT RunConfigurationFactory
 {
-    Q_OBJECT
-
 public:
-    explicit IRunConfigurationFactory(QObject *parent = nullptr);
-    ~IRunConfigurationFactory();
+    RunConfigurationFactory();
+    RunConfigurationFactory(const RunConfigurationFactory &) = delete;
+    RunConfigurationFactory operator=(const RunConfigurationFactory &) = delete;
+    virtual ~RunConfigurationFactory();
 
-    static const QList<IRunConfigurationFactory *> allRunConfigurationFactories();
+    static RunConfiguration *restore(Target *parent, const QVariantMap &map);
+    static RunConfiguration *clone(Target *parent, RunConfiguration *source);
+    static const QList<RunConfigurationCreationInfo> creatorsForTarget(Target *parent);
 
-    enum CreationMode {UserCreate, AutoCreate};
+    Core::Id runConfigurationBaseId() const { return m_runConfigBaseId; }
 
-    QList<RunConfigurationCreationInfo> availableCreators(Target *parent,
-                                                          CreationMode mode = UserCreate) const;
-
-    virtual bool canHandle(Target *target) const;
-
-    RunConfiguration *create(Target *parent, Core::Id id, const QString &extra) const;
-    bool canRestore(Target *parent, const QVariantMap &map) const;
-    RunConfiguration *restore(Target *parent, const QVariantMap &map) const;
-    bool canClone(Target *parent, RunConfiguration *product) const;
-    RunConfiguration *clone(Target *parent, RunConfiguration *product) const;
-
-    static IRunConfigurationFactory *find(Target *parent, const QVariantMap &map);
-    static IRunConfigurationFactory *find(Target *parent, RunConfiguration *rc);
-    static QList<IRunConfigurationFactory *> find(Target *parent);
+    static QString decoratedTargetName(const QString &targetName, Target *kit);
 
 protected:
-    virtual QList<BuildTargetInfo> availableBuildTargets(Target *parent, CreationMode mode = UserCreate) const;
-
-    virtual bool canCreateHelper(Target *parent, const QString &buildTarget) const;
+    virtual QList<RunConfigurationCreationInfo> availableCreators(Target *parent) const;
 
     using RunConfigurationCreator = std::function<RunConfiguration *(Target *)>;
 
     template <class RunConfig>
     void registerRunConfiguration(Core::Id runConfigBaseId)
     {
-        m_creator = [](Target *t) -> RunConfiguration * { return new RunConfig(t); };
+        m_creator = [runConfigBaseId](Target *t) -> RunConfiguration * {
+            return new RunConfig(t, runConfigBaseId);
+        };
         m_runConfigBaseId = runConfigBaseId;
+        m_ownTypeChecker = [](RunConfiguration *runConfig) {
+            return qobject_cast<RunConfig *>(runConfig) != nullptr;
+        };
     }
 
     void addSupportedProjectType(Core::Id id);
-    void setSupportedTargetDeviceTypes(const QList<Core::Id> &ids);
-    void addFixedBuildTarget(const QString &displayName);
-    void setDisplayNamePattern(const QString &pattern);
+    void addSupportedTargetDeviceType(Core::Id id);
+    void setDecorateDisplayNames(bool on);
+
+    template<class Worker>
+    RunWorkerFactory *addRunWorkerFactory(Core::Id runMode)
+    {
+        return addRunWorkerFactoryHelper(runMode, [](RunControl *rc) { return new Worker(rc); });
+    }
 
 private:
+    RunWorkerFactory *addRunWorkerFactoryHelper
+        (Core::Id runMode, const std::function<RunWorker *(RunControl *)> &creator);
+
+    bool canHandle(Target *target) const;
+
+    friend class RunConfigurationCreationInfo;
     RunConfigurationCreator m_creator;
     Core::Id m_runConfigBaseId;
     QList<Core::Id> m_supportedProjectTypes;
     QList<Core::Id> m_supportedTargetDeviceTypes;
-    QList<BuildTargetInfo> m_fixedBuildTargets;
-    QString m_displayNamePattern;
+    bool m_decorateDisplayNames = false;
+    QList<RunWorkerFactory *> m_ownedRunWorkerFactories;
+    std::function<bool(RunConfiguration *)> m_ownTypeChecker;
 };
 
-class PROJECTEXPLORER_EXPORT RunConfigWidget : public QWidget
+class PROJECTEXPLORER_EXPORT FixedRunConfigurationFactory : public RunConfigurationFactory
 {
-    Q_OBJECT
-
 public:
-    virtual QString displayName() const = 0;
+    explicit FixedRunConfigurationFactory(const QString &displayName,
+                                          bool addDeviceName = false);
 
-signals:
-    void displayNameChanged(const QString &);
+    QList<RunConfigurationCreationInfo> availableCreators(Target *parent) const override;
+
+private:
+    const QString m_fixedBuildTarget;
+    const bool m_decorateTargetName;
 };
 
 class PROJECTEXPLORER_EXPORT RunWorker : public QObject
@@ -369,7 +324,6 @@ public:
     void addStartDependency(RunWorker *dependency);
     void addStopDependency(RunWorker *dependency);
 
-    void setDisplayName(const QString &id) { setId(id); } // FIXME: Obsoleted by setId.
     void setId(const QString &id);
 
     void setStartTimeout(int ms, const std::function<void()> &callback = {});
@@ -414,7 +368,35 @@ protected:
 private:
     friend class Internal::RunControlPrivate;
     friend class Internal::RunWorkerPrivate;
-    Internal::RunWorkerPrivate *d;
+    const std::unique_ptr<Internal::RunWorkerPrivate> d;
+};
+
+class PROJECTEXPLORER_EXPORT RunWorkerFactory
+{
+public:
+    using WorkerCreator = std::function<RunWorker *(RunControl *)>;
+    using Constraint = std::function<bool(RunConfiguration *)>;
+
+    RunWorkerFactory();
+    virtual ~RunWorkerFactory();
+
+    bool canRun(RunConfiguration *runConfiguration, Core::Id runMode) const;
+
+    void setProducer(const WorkerCreator &producer);
+    void addConstraint(const Constraint &constraint);
+    void addSupportedRunMode(Core::Id runMode);
+
+    WorkerCreator producer() const { return m_producer; }
+
+private:
+    // FIXME: That's temporary until ownership has been transferred to
+    // the individual plugins.
+    friend class ProjectExplorerPlugin;
+    static void destroyRemainingRunWorkerFactories();
+
+    QList<Core::Id> m_supportedRunModes;
+    QList<Constraint> m_constraints;
+    WorkerCreator m_producer;
 };
 
 /**
@@ -431,6 +413,7 @@ class PROJECTEXPLORER_EXPORT RunControl : public QObject
 
 public:
     RunControl(RunConfiguration *runConfiguration, Core::Id mode);
+    RunControl(const IDevice::ConstPtr &device, Core::Id mode);
     ~RunControl() override;
 
     void initiateStart();
@@ -457,12 +440,10 @@ public:
 
     Utils::ProcessHandle applicationProcessHandle() const;
     void setApplicationProcessHandle(const Utils::ProcessHandle &handle);
-    Abi abi() const;
     IDevice::ConstPtr device() const;
 
     RunConfiguration *runConfiguration() const;
     Project *project() const;
-    bool canReUseOutputPane(const RunControl *other) const;
 
     Utils::OutputFormatter *outputFormatter() const;
     Core::Id runMode() const;
@@ -479,41 +460,27 @@ public:
 
     RunWorker *createWorker(Core::Id id);
 
-    using WorkerCreator = std::function<RunWorker *(RunControl *)>;
-    using Constraint = std::function<bool(RunConfiguration *)>;
+    using WorkerCreator = RunWorkerFactory::WorkerCreator;
+    using Constraint = RunWorkerFactory::Constraint;
 
     static void registerWorkerCreator(Core::Id id, const WorkerCreator &workerCreator);
 
     static void registerWorker(Core::Id runMode, const WorkerCreator &producer,
                                const Constraint &constraint = {})
     {
-        addWorkerFactory({runMode, constraint, producer});
+        auto factory = new RunWorkerFactory;
+        factory->setProducer(producer);
+        factory->addSupportedRunMode(runMode);
+        factory->addConstraint(constraint);
     }
     template <class Worker>
-    static void registerWorker(Core::Id runMode, const Constraint &constraint, int priority = 0)
+    static void registerWorker(Core::Id runMode, const Constraint &constraint)
     {
-        auto producer = [](RunControl *rc) { return new Worker(rc); };
-        addWorkerFactory({runMode, constraint, producer, priority});
+        auto factory = new RunWorkerFactory;
+        factory->setProducer([](RunControl *rc) { return new Worker(rc); });
+        factory->addSupportedRunMode(runMode);
+        factory->addConstraint(constraint);
     }
-    template <class Config, class Worker>
-    static void registerWorker(Core::Id runMode, int priority = 0)
-    {
-        auto constraint = [](RunConfiguration *runConfig) { return qobject_cast<Config *>(runConfig) != nullptr; };
-        auto producer = [](RunControl *rc) { return new Worker(rc); };
-        addWorkerFactory({runMode, constraint, producer, priority});
-    }
-
-    struct WorkerFactory {
-        Core::Id runMode;
-        Constraint constraint;
-        WorkerCreator producer;
-        int priority = 0;
-
-        WorkerFactory(const Core::Id &mode, Constraint constr, const WorkerCreator &prod,
-                      int prio = 0)
-            : runMode(mode), constraint(constr), producer(prod), priority(prio) {}
-        bool canRun(RunConfiguration *runConfiguration, Core::Id runMode) const;
-    };
 
     static WorkerCreator producer(RunConfiguration *runConfiguration, Core::Id runMode);
 
@@ -530,8 +497,7 @@ private:
     friend class RunWorker;
     friend class Internal::RunWorkerPrivate;
 
-    static void addWorkerFactory(const WorkerFactory &workerFactory);
-    Internal::RunControlPrivate *d;
+    const std::unique_ptr<Internal::RunControlPrivate> d;
 };
 
 
@@ -565,6 +531,7 @@ private:
     Runnable m_runnable;
     IDevice::ConstPtr m_device;
     bool m_stopReported = false;
+    bool m_useTerminal = false;
 };
 
 } // namespace ProjectExplorer
