@@ -74,9 +74,9 @@ namespace Internal {
 class TaskView : public Utils::ListView
 {
 public:
-    TaskView(QWidget *parent = 0);
-    ~TaskView();
-    void resizeEvent(QResizeEvent *e);
+    TaskView(QWidget *parent = nullptr);
+    ~TaskView() override;
+    void resizeEvent(QResizeEvent *e) override;
 };
 
 class TaskWindowContext : public Core::IContext
@@ -92,10 +92,10 @@ class TaskDelegate : public QStyledItemDelegate
     friend class TaskView; // for using Positions::minimumSize()
 
 public:
-    TaskDelegate(QObject * parent = 0);
-    ~TaskDelegate();
-    void paint(QPainter *painter, const QStyleOptionViewItem &option, const QModelIndex &index) const;
-    QSize sizeHint(const QStyleOptionViewItem &option, const QModelIndex &index) const;
+    TaskDelegate(QObject * parent = nullptr);
+    ~TaskDelegate() override;
+    void paint(QPainter *painter, const QStyleOptionViewItem &option, const QModelIndex &index) const override;
+    QSize sizeHint(const QStyleOptionViewItem &option, const QModelIndex &index) const override;
 
     // TaskView uses this method if the size of the taskview changes
     void emitSizeHintChanged(const QModelIndex &index);
@@ -105,7 +105,7 @@ public:
 private:
     void generateGradientPixmap(int width, int height, QColor color, bool selected) const;
 
-    mutable int m_cachedHeight;
+    mutable int m_cachedHeight = 0;
     mutable QFont m_cachedFont;
 
     /*
@@ -194,8 +194,7 @@ TaskView::TaskView(QWidget *parent)
     verticalScrollBar()->setSingleStep(vStepSize);
 }
 
-TaskView::~TaskView()
-{ }
+TaskView::~TaskView() = default;
 
 void TaskView::resizeEvent(QResizeEvent *e)
 {
@@ -243,7 +242,7 @@ static QToolButton *createFilterButton(const QIcon &icon, const QString &toolTip
     return button;
 }
 
-TaskWindow::TaskWindow() : d(new TaskWindowPrivate)
+TaskWindow::TaskWindow() : d(std::make_unique<TaskWindowPrivate>())
 {
     d->m_model = new Internal::TaskModel(this);
     d->m_filter = new Internal::TaskFilterModel(d->m_model);
@@ -253,7 +252,7 @@ TaskWindow::TaskWindow() : d(new TaskWindowPrivate)
     d->m_listview->setFrameStyle(QFrame::NoFrame);
     d->m_listview->setWindowTitle(displayName());
     d->m_listview->setSelectionMode(QAbstractItemView::SingleSelection);
-    Internal::TaskDelegate *tld = new Internal::TaskDelegate(this);
+    auto *tld = new Internal::TaskDelegate(this);
     d->m_listview->setItemDelegate(tld);
     d->m_listview->setWindowIcon(Icons::WINDOW.icon());
     d->m_listview->setContextMenuPolicy(Qt::ActionsContextMenu);
@@ -322,7 +321,6 @@ TaskWindow::~TaskWindow()
     delete d->m_listview;
     delete d->m_filter;
     delete d->m_model;
-    delete d;
 }
 
 void TaskWindow::delayedInitialization()
@@ -345,8 +343,8 @@ void TaskWindow::delayedInitialization()
 
         Core::Id id = h->actionManagerId();
         if (id.isValid()) {
-            Core::Command *cmd = Core::ActionManager::instance()
-                    ->registerAction(action, id, d->m_taskWindowContext->context(), true);
+            Core::Command *cmd =
+                Core::ActionManager::registerAction(action, id, d->m_taskWindowContext->context(), true);
             action = cmd->action();
         }
         d->m_listview->addAction(action);
@@ -446,9 +444,12 @@ void TaskWindow::addTask(const Task &task)
     emit tasksChanged();
     navigateStateChanged();
 
-    if (task.type == Task::Error && d->m_filter->filterIncludesErrors()
-            && !d->m_filter->filteredCategories().contains(task.category))
+    if ((task.options & Task::FlashWorthy)
+         && task.type == Task::Error
+         && d->m_filter->filterIncludesErrors()
+         && !d->m_filter->filteredCategories().contains(task.category)) {
         flash();
+    }
 }
 
 void TaskWindow::removeTask(const Task &task)
@@ -530,7 +531,7 @@ void TaskWindow::setShowWarnings(bool show)
 
 void TaskWindow::updateCategoriesMenu()
 {
-    typedef QMap<QString, Core::Id>::ConstIterator NameToIdsConstIt;
+    using NameToIdsConstIt = QMap<QString, Core::Id>::ConstIterator;
 
     d->m_categoriesMenu->clear();
 
@@ -667,13 +668,10 @@ bool TaskWindow::canNavigate() const
 /////
 
 TaskDelegate::TaskDelegate(QObject *parent) :
-    QStyledItemDelegate(parent),
-    m_cachedHeight(0)
+    QStyledItemDelegate(parent)
 { }
 
-TaskDelegate::~TaskDelegate()
-{
-}
+TaskDelegate::~TaskDelegate() = default;
 
 QSize TaskDelegate::sizeHint(const QStyleOptionViewItem &option, const QModelIndex &index) const
 {
@@ -760,8 +758,8 @@ void TaskDelegate::paint(QPainter *painter, const QStyleOptionViewItem &option,
         painter->setBrush(opt.palette.highlight().color());
         backgroundColor = opt.palette.highlight().color();
     } else {
-        painter->setBrush(opt.palette.background().color());
-        backgroundColor = opt.palette.background().color();
+        painter->setBrush(opt.palette.window().color());
+        backgroundColor = opt.palette.window().color();
     }
     painter->setPen(Qt::NoPen);
     painter->drawRect(opt.rect);
