@@ -25,28 +25,40 @@
 
 #include "exampleslistmodel.h"
 
+#include "screenshotcropper.h"
+
+#include <QBuffer>
 #include <QDebug>
 #include <QDir>
 #include <QFile>
+#include <QFutureWatcher>
+#include <QImageReader>
+#include <QPixmapCache>
 #include <QUrl>
 #include <QXmlStreamReader>
 
 #include <coreplugin/helpmanager.h>
 #include <coreplugin/icore.h>
+
 #include <qtsupport/qtkitinformation.h>
 #include <qtsupport/qtversionmanager.h>
-#include <utils/environment.h>
-#include <utils/qtcassert.h>
+
 #include <utils/algorithm.h>
+#include <utils/environment.h>
+#include <utils/fileutils.h>
+#include <utils/qtcassert.h>
+#include <utils/stylehelper.h>
 
 #include <algorithm>
 
 namespace QtSupport {
 namespace Internal {
 
+const QSize ExamplesListModel::exampleImageSize(188, 145);
+
 static bool debugExamples()
 {
-    static bool isDebugging = !qgetenv("QTC_DEBUG_EXAMPLESMODEL").isEmpty();
+    static bool isDebugging = qEnvironmentVariableIsSet("QTC_DEBUG_EXAMPLESMODEL");
     return isDebugging;
 }
 
@@ -68,22 +80,57 @@ int ExampleSetModel::readCurrentIndexFromSettings() const
     return -1;
 }
 
-ExampleSetModel::ExampleSetModel(ExamplesListModel *examplesModel, QObject *parent) :
-    QStandardItemModel(parent),
-    examplesModel(examplesModel)
+ExampleSetModel::ExampleSetModel()
 {
+    // read extra example sets settings
+    QSettings *settings = Core::ICore::settings();
+    const QStringList list = settings->value("Help/InstalledExamples", QStringList()).toStringList();
+    if (debugExamples())
+        qWarning() << "Reading Help/InstalledExamples from settings:" << list;
+    for (const QString &item : list) {
+        const QStringList &parts = item.split(QLatin1Char('|'));
+        if (parts.size() < 3) {
+            if (debugExamples())
+                qWarning() << "Item" << item << "has less than 3 parts (separated by '|'):" << parts;
+            continue;
+        }
+        ExtraExampleSet set;
+        set.displayName = parts.at(0);
+        set.manifestPath = parts.at(1);
+        set.examplesPath = parts.at(2);
+        QFileInfo fi(set.manifestPath);
+        if (!fi.isDir() || !fi.isReadable()) {
+            if (debugExamples())
+                qWarning() << "Manifest path " << set.manifestPath << "is not a readable directory, ignoring";
+            continue;
+        }
+        m_extraExampleSets.append(set);
+        if (debugExamples()) {
+            qWarning() << "Adding examples set displayName=" << set.displayName
+                       << ", manifestPath=" << set.manifestPath
+                       << ", examplesPath=" << set.examplesPath;
+        }
+    }
+
+    connect(QtVersionManager::instance(), &QtVersionManager::qtVersionsLoaded,
+            this, &ExampleSetModel::qtVersionManagerLoaded);
+
+    connect(Core::HelpManager::Signals::instance(),
+            &Core::HelpManager::Signals::setupFinished,
+            this,
+            &ExampleSetModel::helpManagerInitialized);
 }
 
-void ExampleSetModel::update()
+void ExampleSetModel::recreateModel(const QList<BaseQtVersion *> &qtVersions)
 {
     beginResetModel();
     clear();
 
     QSet<QString> extraManifestDirs;
-    QList<ExamplesListModel::ExtraExampleSet> extraExampleSets = examplesModel->extraExampleSets();
-    for (int i = 0; i < extraExampleSets.size(); ++i)  {
-        ExamplesListModel::ExtraExampleSet set = extraExampleSets.at(i);
-        QStandardItem *newItem = new QStandardItem();
+    for (int i = 0; i < m_extraExampleSets.size(); ++i)  {
+        const ExtraExampleSet &set = m_extraExampleSets.at(i);
+        auto newItem = new QStandardItem();
+        newItem->setData(set.displayName, Qt::DisplayRole);
         newItem->setData(set.displayName, Qt::UserRole + 1);
         newItem->setData(QVariant(), Qt::UserRole + 2);
         newItem->setData(i, Qt::UserRole + 3);
@@ -92,8 +139,6 @@ void ExampleSetModel::update()
         extraManifestDirs.insert(set.manifestPath);
     }
 
-    QList<BaseQtVersion *> qtVersions = examplesModel->qtVersions();
-
     foreach (BaseQtVersion *version, qtVersions) {
         // sanitize away qt versions that have already been added through extra sets
         if (extraManifestDirs.contains(version->documentationPath())) {
@@ -103,7 +148,8 @@ void ExampleSetModel::update()
             }
             continue;
         }
-        QStandardItem *newItem = new QStandardItem();
+        auto newItem = new QStandardItem();
+        newItem->setData(version->displayName(), Qt::DisplayRole);
         newItem->setData(version->displayName(), Qt::UserRole + 1);
         newItem->setData(version->uniqueId(), Qt::UserRole + 2);
         newItem->setData(QVariant(), Qt::UserRole + 3);
@@ -124,13 +170,12 @@ int ExampleSetModel::indexForQtVersion(BaseQtVersion *qtVersion) const
         if (getType(i) == QtExampleSet && getQtId(i) == qtVersion->uniqueId())
             return i;
     }
+
     // check for extra set
-    const QList<ExamplesListModel::ExtraExampleSet> &extraExamples
-            = examplesModel->extraExampleSets();
     const QString &documentationPath = qtVersion->documentationPath();
     for (int i = 0; i < rowCount(); ++i) {
-        if (getType(i) == ExtraExampleSet
-                && extraExamples.at(getExtraExampleSetIndex(i)).manifestPath == documentationPath)
+        if (getType(i) == ExtraExampleSetType
+                && m_extraExampleSets.at(getExtraExampleSetIndex(i)).manifestPath == documentationPath)
             return i;
     }
     return -1;
@@ -163,7 +208,7 @@ ExampleSetModel::ExampleSetType ExampleSetModel::getType(int i) const
     QVariant variant = data(modelIndex, Qt::UserRole + 2); /*Qt version uniqueId*/
     if (variant.isValid())
         return QtExampleSet;
-    return ExtraExampleSet;
+    return ExtraExampleSetType;
 }
 
 int ExampleSetModel::getQtId(int i) const
@@ -186,51 +231,15 @@ int ExampleSetModel::getExtraExampleSetIndex(int i) const
     return variant.toInt();
 }
 
-QHash<int, QByteArray> ExampleSetModel::roleNames() const
+ExamplesListModel::ExamplesListModel(QObject *parent)
+    : QAbstractListModel(parent)
 {
-    static QHash<int, QByteArray> roleNames{
-        {Qt::UserRole + 1, "text"},
-        {Qt::UserRole + 2, "QtId"},
-        {Qt::UserRole + 3, "extraSetIndex"}
-    };
-    return roleNames;
-}
-
-ExamplesListModel::ExamplesListModel(QObject *parent) :
-    QAbstractListModel(parent),
-    m_exampleSetModel(new ExampleSetModel(this, this)),
-    m_selectedExampleSetIndex(-1)
-{
-    // read extra example sets settings
-    QSettings *settings = Core::ICore::settings();
-    QStringList list = settings->value(QLatin1String("Help/InstalledExamples"),
-                                       QStringList()).toStringList();
-    if (debugExamples())
-        qWarning() << "Reading Help/InstalledExamples from settings:" << list;
-    foreach (const QString &item, list) {
-        const QStringList &parts = item.split(QLatin1Char('|'));
-        if (parts.size() < 3) {
-            if (debugExamples())
-                qWarning() << "Item" << item << "has less than 3 parts (separated by '|'):" << parts;
-            continue;
-        }
-        ExtraExampleSet set;
-        set.displayName = parts.at(0);
-        set.manifestPath = parts.at(1);
-        set.examplesPath = parts.at(2);
-        QFileInfo fi(set.manifestPath);
-        if (!fi.isDir() || !fi.isReadable()) {
-            if (debugExamples())
-                qWarning() << "Manifest path " << set.manifestPath << "is not a readable directory, ignoring";
-            continue;
-        }
-        m_extraExampleSets.append(set);
-        if (debugExamples()) {
-            qWarning() << "Adding examples set displayName=" << set.displayName
-                       << ", manifestPath=" << set.manifestPath
-                       << ", examplesPath=" << set.examplesPath;
-        }
-    }
+    connect(&m_exampleSetModel, &ExampleSetModel::selectedExampleSetChanged,
+            this, &ExamplesListModel::updateExamples);
+    connect(Core::HelpManager::Signals::instance(),
+            &Core::HelpManager::Signals::documentationChanged,
+            this,
+            &ExamplesListModel::updateExamples);
 }
 
 static QString fixStringForTags(const QString &string)
@@ -306,10 +315,9 @@ void ExamplesListModel::parseExamples(QXmlStreamReader *reader,
                 item.hasSourceCode = !item.projectPath.isEmpty();
                 item.projectPath = relativeOrInstallPath(item.projectPath, projectsOffset, examplesInstallPath);
                 item.imageUrl = attributes.value(QLatin1String("imageUrl")).toString();
+                QPixmapCache::remove(item.imageUrl);
                 item.docUrl = attributes.value(QLatin1String("docUrl")).toString();
-
-                if (attributes.hasAttribute(QLatin1String("isHighlighted")))
-                    item.isHighlighted = attributes.value(QLatin1String("isHighlighted")).toString() == QLatin1String("true");
+                item.isHighlighted = attributes.value(QLatin1String("isHighlighted")).toString() == QLatin1String("true");
 
             } else if (reader->name() == QLatin1String("fileToOpen")) {
                 const QString mainFileAttribute = reader->attributes().value(
@@ -361,11 +369,9 @@ void ExamplesListModel::parseDemos(QXmlStreamReader *reader,
                 item.hasSourceCode = !item.projectPath.isEmpty();
                 item.projectPath = relativeOrInstallPath(item.projectPath, projectsOffset, demosInstallPath);
                 item.imageUrl = attributes.value(QLatin1String("imageUrl")).toString();
+                QPixmapCache::remove(item.imageUrl);
                 item.docUrl = attributes.value(QLatin1String("docUrl")).toString();
-
-                if (attributes.hasAttribute(QLatin1String("isHighlighted")))
-                    item.isHighlighted = attributes.value(QLatin1String("isHighlighted")).toString() == QLatin1String("true");
-
+                item.isHighlighted = attributes.value(QLatin1String("isHighlighted")).toString() == QLatin1String("true");
             } else if (reader->name() == QLatin1String("fileToOpen")) {
                 item.filesToOpen.append(relativeOrInstallPath(reader->readElementText(QXmlStreamReader::ErrorOnUnexpectedElement),
                                                               projectsOffset, demosInstallPath));
@@ -407,16 +413,13 @@ void ExamplesListModel::parseTutorials(QXmlStreamReader *reader, const QString &
                 item.hasSourceCode = !item.projectPath.isEmpty();
                 item.projectPath.prepend(slash);
                 item.projectPath.prepend(projectsOffset);
-                if (attributes.hasAttribute(QLatin1String("imageUrl")))
-                    item.imageUrl = attributes.value(QLatin1String("imageUrl")).toString();
-                if (attributes.hasAttribute(QLatin1String("docUrl")))
-                    item.docUrl = attributes.value(QLatin1String("docUrl")).toString();
-                if (attributes.hasAttribute(QLatin1String("isVideo")))
-                    item.isVideo = attributes.value(QLatin1String("isVideo")).toString() == QLatin1String("true");
-                if (attributes.hasAttribute(QLatin1String("videoUrl")))
-                    item.videoUrl = attributes.value(QLatin1String("videoUrl")).toString();
-                if (attributes.hasAttribute(QLatin1String("videoLength")))
-                    item.videoLength = attributes.value(QLatin1String("videoLength")).toString();
+                item.imageUrl = Utils::StyleHelper::dpiSpecificImageFile(
+                            attributes.value(QLatin1String("imageUrl")).toString());
+                QPixmapCache::remove(item.imageUrl);
+                item.docUrl = attributes.value(QLatin1String("docUrl")).toString();
+                item.isVideo = attributes.value(QLatin1String("isVideo")).toString() == QLatin1String("true");
+                item.videoUrl = attributes.value(QLatin1String("videoUrl")).toString();
+                item.videoLength = attributes.value(QLatin1String("videoLength")).toString();
             } else if (reader->name() == QLatin1String("fileToOpen")) {
                 item.filesToOpen.append(projectsOffset + slash + reader->readElementText(QXmlStreamReader::ErrorOnUnexpectedElement));
             } else if (reader->name() == QLatin1String("description")) {
@@ -439,12 +442,18 @@ void ExamplesListModel::parseTutorials(QXmlStreamReader *reader, const QString &
     }
 }
 
+static QString resourcePath()
+{
+    // normalize paths so QML doesn't freak out if it's wrongly capitalized on Windows
+    return Utils::FileUtils::normalizePathName(Core::ICore::resourcePath());
+}
+
 void ExamplesListModel::updateExamples()
 {
     QString examplesInstallPath;
     QString demosInstallPath;
 
-    QStringList sources = exampleSources(&examplesInstallPath, &demosInstallPath);
+    QStringList sources = m_exampleSetModel.exampleSources(&examplesInstallPath, &demosInstallPath);
 
     beginResetModel();
     m_exampleItems.clear();
@@ -485,17 +494,13 @@ void ExamplesListModel::updateExamples()
     endResetModel();
 }
 
-void ExamplesListModel::updateQtVersions()
+void ExampleSetModel::updateQtVersionList()
 {
-    QList<BaseQtVersion*> versions = QtVersionManager::validVersions();
-
-    QMutableListIterator<BaseQtVersion*> iter(versions);
-    while (iter.hasNext()) {
-        BaseQtVersion *version = iter.next();
-        if (!version->hasExamples()
-                && !version->hasDemos())
-            iter.remove();
-    }
+    QList<BaseQtVersion*> versions
+            = QtVersionManager::sortVersions(
+                QtVersionManager::versions(BaseQtVersion::isValidPredicate([](const BaseQtVersion *v) {
+        return v->hasExamples() || v->hasDemos();
+    })));
 
     // prioritize default qt version
     ProjectExplorer::Kit *defaultKit = ProjectExplorer::KitManager::defaultKit();
@@ -503,45 +508,35 @@ void ExamplesListModel::updateQtVersions()
     if (defaultVersion && versions.contains(defaultVersion))
         versions.move(versions.indexOf(defaultVersion), 0);
 
-    if (m_qtVersions == versions && m_selectedExampleSetIndex >= 0)
-        return;
-
-    m_qtVersions = versions;
-
-    m_exampleSetModel->update();
+    recreateModel(versions);
 
     int currentIndex = m_selectedExampleSetIndex;
     if (currentIndex < 0) // reset from settings
-        currentIndex = m_exampleSetModel->readCurrentIndexFromSettings();
+        currentIndex = readCurrentIndexFromSettings();
 
-    ExampleSetModel::ExampleSetType currentType
-            = m_exampleSetModel->getType(currentIndex);
+    ExampleSetModel::ExampleSetType currentType = getType(currentIndex);
 
     if (currentType == ExampleSetModel::InvalidExampleSet) {
         // select examples corresponding to 'highest' Qt version
-        BaseQtVersion *highestQt = findHighestQtVersion();
-        currentIndex = m_exampleSetModel->indexForQtVersion(highestQt);
+        BaseQtVersion *highestQt = findHighestQtVersion(versions);
+        currentIndex = indexForQtVersion(highestQt);
     } else if (currentType == ExampleSetModel::QtExampleSet) {
         // try to select the previously selected Qt version, or
         // select examples corresponding to 'highest' Qt version
-        int currentQtId = m_exampleSetModel->getQtId(currentIndex);
-        BaseQtVersion *newQtVersion = Utils::findOrDefault(m_qtVersions,
-                                                    Utils::equal(&BaseQtVersion::uniqueId, currentQtId));
-
+        int currentQtId = getQtId(currentIndex);
+        BaseQtVersion *newQtVersion = QtVersionManager::version(currentQtId);
         if (!newQtVersion)
-            newQtVersion = findHighestQtVersion();
-        currentIndex = m_exampleSetModel->indexForQtVersion(newQtVersion);
+            newQtVersion = findHighestQtVersion(versions);
+        currentIndex = indexForQtVersion(newQtVersion);
     } // nothing to do for extra example sets
     selectExampleSet(currentIndex);
+    emit selectedExampleSetChanged(currentIndex);
 }
 
-BaseQtVersion *ExamplesListModel::findHighestQtVersion() const
+BaseQtVersion *ExampleSetModel::findHighestQtVersion(const QList<BaseQtVersion *> &versions) const
 {
-    QList<BaseQtVersion *> versions = qtVersions();
-
-    BaseQtVersion *newVersion = 0;
-
-    foreach (BaseQtVersion *version, versions) {
+    BaseQtVersion *newVersion = nullptr;
+    for (BaseQtVersion *version : versions) {
         if (!newVersion) {
             newVersion = version;
         } else {
@@ -557,35 +552,30 @@ BaseQtVersion *ExamplesListModel::findHighestQtVersion() const
     if (!newVersion && !versions.isEmpty())
         newVersion = versions.first();
 
-    if (!newVersion)
-        return 0;
-
     return newVersion;
 }
 
-QStringList ExamplesListModel::exampleSources(QString *examplesInstallPath, QString *demosInstallPath)
+QStringList ExampleSetModel::exampleSources(QString *examplesInstallPath, QString *demosInstallPath)
 {
     QStringList sources;
-    QString resourceDir = Core::ICore::resourcePath() + QLatin1String("/welcomescreen/");
 
     // Qt Creator shipped tutorials
-    sources << (resourceDir + QLatin1String("/qtcreator_tutorials.xml"));
+    sources << ":/qtsupport/qtcreator_tutorials.xml";
 
     QString examplesPath;
     QString demosPath;
     QString manifestScanPath;
 
-    ExampleSetModel::ExampleSetType currentType
-            = m_exampleSetModel->getType(m_selectedExampleSetIndex);
-    if (currentType == ExampleSetModel::ExtraExampleSet) {
-        int index = m_exampleSetModel->getExtraExampleSetIndex(m_selectedExampleSetIndex);
+    ExampleSetModel::ExampleSetType currentType = getType(m_selectedExampleSetIndex);
+    if (currentType == ExampleSetModel::ExtraExampleSetType) {
+        int index = getExtraExampleSetIndex(m_selectedExampleSetIndex);
         ExtraExampleSet exampleSet = m_extraExampleSets.at(index);
         manifestScanPath = exampleSet.manifestPath;
         examplesPath = exampleSet.examplesPath;
         demosPath = exampleSet.examplesPath;
     } else if (currentType == ExampleSetModel::QtExampleSet) {
-        int qtId = m_exampleSetModel->getQtId(m_selectedExampleSetIndex);
-        foreach (BaseQtVersion *version, qtVersions()) {
+        int qtId = getQtId(m_selectedExampleSetIndex);
+        foreach (BaseQtVersion *version, QtVersionManager::versions()) {
             if (version->uniqueId() == qtId) {
                 manifestScanPath = version->documentationPath();
                 examplesPath = version->examplesPath();
@@ -629,179 +619,123 @@ QString prefixForItem(const ExampleItem &item)
 
 QVariant ExamplesListModel::data(const QModelIndex &index, int role) const
 {
-    if (!index.isValid() || index.row()+1 > m_exampleItems.count()) {
-        qDebug() << Q_FUNC_INFO << "invalid index requested";
+    if (!index.isValid() || index.row() >= m_exampleItems.count())
         return QVariant();
-    }
 
-    ExampleItem item = m_exampleItems.at(index.row());
+    const ExampleItem &item = m_exampleItems.at(index.row());
     switch (role)
     {
     case Qt::DisplayRole: // for search only
-        return QString(prefixForItem(item) + item.name + QLatin1Char(' ') + item.tags.join(QLatin1Char(' ')));
-    case Name:
-        return item.name;
-    case ProjectPath:
-        return item.projectPath;
-    case Description:
-        return item.description;
-    case ImageUrl:
-        return item.imageUrl;
-    case DocUrl:
-        return item.docUrl;
-    case FilesToOpen:
-        return item.filesToOpen;
-    case MainFile:
-        return item.mainFile;
-    case Tags:
-        return item.tags;
-    case Difficulty:
-        return item.difficulty;
-    case Dependencies:
-        return item.dependencies;
-    case HasSourceCode:
-        return item.hasSourceCode;
-    case Type:
-        return item.type;
-    case IsVideo:
-        return item.isVideo;
-    case VideoUrl:
-        return item.videoUrl;
-    case VideoLength:
-        return item.videoLength;
-    case Platforms:
-        return item.platforms;
-    case IsHighlighted:
-        return item.isHighlighted;
+        return QString(prefixForItem(item) + item.name + ' ' + item.tags.join(' '));
+    case ExampleItemRole:
+        return QVariant::fromValue<ExampleItem>(item);
+    case ExampleImageRole: {
+        QPixmap pixmap;
+        if (QPixmapCache::find(item.imageUrl, &pixmap))
+            return pixmap;
+        pixmap.load(item.imageUrl);
+        if (pixmap.isNull())
+            pixmap.load(resourcePath() + "/welcomescreen/widgets/" + item.imageUrl);
+        if (pixmap.isNull()) {
+            QByteArray fetchedData = Core::HelpManager::fileData(item.imageUrl);
+            if (!fetchedData.isEmpty()) {
+                QBuffer imgBuffer(&fetchedData);
+                imgBuffer.open(QIODevice::ReadOnly);
+                QImageReader reader(&imgBuffer);
+                QImage img = reader.read();
+                img = ScreenshotCropper::croppedImage(img, item.imageUrl,
+                                                      ExamplesListModel::exampleImageSize);
+                pixmap = QPixmap::fromImage(img);
+            }
+        }
+        QPixmapCache::insert(item.imageUrl, pixmap);
+        return pixmap;
+    }
     default:
-        qDebug() << Q_FUNC_INFO << "role type not supported";
         return QVariant();
     }
 }
 
-QHash<int, QByteArray> ExamplesListModel::roleNames() const
-{
-    static QHash<int, QByteArray> roleNames{
-        {Name, "name"},
-        {ProjectPath, "projectPath"},
-        {ImageUrl, "imageUrl"},
-        {Description, "description"},
-        {DocUrl, "docUrl"},
-        {FilesToOpen, "filesToOpen"},
-        {MainFile, "mainFile"},
-        {Tags, "tags"},
-        {Difficulty, "difficulty"},
-        {Type, "type"},
-        {HasSourceCode, "hasSourceCode"},
-        {Dependencies, "dependencies"},
-        {IsVideo, "isVideo"},
-        {VideoUrl, "videoUrl"},
-        {VideoLength, "videoLength"},
-        {Platforms, "platforms"},
-        {IsHighlighted, "isHighlighted"}
-    };
-    return roleNames;
-}
-
-void ExamplesListModel::update()
-{
-    updateQtVersions();
-    updateExamples();
-}
-
-int ExamplesListModel::selectedExampleSet() const
-{
-    return m_selectedExampleSetIndex;
-}
-
-void ExamplesListModel::selectExampleSet(int index)
+void ExampleSetModel::selectExampleSet(int index)
 {
     if (index != m_selectedExampleSetIndex) {
         m_selectedExampleSetIndex = index;
-        m_exampleSetModel->writeCurrentIdToSettings(m_selectedExampleSetIndex);
-        updateExamples();
-        emit selectedExampleSetChanged();
+        writeCurrentIdToSettings(m_selectedExampleSetIndex);
+        emit selectedExampleSetChanged(m_selectedExampleSetIndex);
     }
 }
 
-ExamplesListModelFilter::ExamplesListModelFilter(ExamplesListModel *sourceModel, QObject *parent) :
+void ExampleSetModel::qtVersionManagerLoaded()
+{
+    m_qtVersionManagerInitialized = true;
+    tryToInitialize();
+}
+
+void ExampleSetModel::helpManagerInitialized()
+{
+    m_helpManagerInitialized = true;
+    tryToInitialize();
+}
+
+
+void ExampleSetModel::tryToInitialize()
+{
+    if (m_initalized)
+        return;
+    if (!m_qtVersionManagerInitialized)
+        return;
+    if (!m_helpManagerInitialized)
+        return;
+
+    m_initalized = true;
+
+    connect(QtVersionManager::instance(), &QtVersionManager::qtVersionsChanged,
+            this, &ExampleSetModel::updateQtVersionList);
+    connect(ProjectExplorer::KitManager::instance(), &ProjectExplorer::KitManager::defaultkitChanged,
+            this, &ExampleSetModel::updateQtVersionList);
+
+    updateQtVersionList();
+}
+
+
+ExamplesListModelFilter::ExamplesListModelFilter(ExamplesListModel *sourceModel, bool showTutorialsOnly, QObject *parent) :
     QSortFilterProxyModel(parent),
-    m_showTutorialsOnly(true),
-    m_sourceModel(sourceModel),
-    m_timerId(0),
-    m_blockIndexUpdate(false),
-    m_qtVersionManagerInitialized(false),
-    m_helpManagerInitialized(false),
-    m_initalized(false),
-    m_exampleDataRequested(false)
+    m_showTutorialsOnly(showTutorialsOnly)
 {
-    // initialization hooks
-    connect(QtVersionManager::instance(), &QtVersionManager::qtVersionsLoaded,
-            this, &ExamplesListModelFilter::qtVersionManagerLoaded);
-    connect(Core::HelpManager::instance(), &Core::HelpManager::setupFinished,
-            this, &ExamplesListModelFilter::helpManagerInitialized);
-
-    connect(this, &ExamplesListModelFilter::showTutorialsOnlyChanged,
-            this, &ExamplesListModelFilter::updateFilter);
-
-    connect(m_sourceModel, &ExamplesListModel::selectedExampleSetChanged,
-            this, &ExamplesListModelFilter::exampleSetIndexChanged);
-
-    setSourceModel(m_sourceModel);
-}
-
-void ExamplesListModelFilter::updateFilter()
-{
-    ExamplesListModel *exampleListModel = qobject_cast<ExamplesListModel*>(sourceModel());
-    if (exampleListModel) {
-        exampleListModel->beginReset();
-        invalidateFilter();
-        exampleListModel->endReset();
-    }
-}
-
-bool containsSubString(const QStringList &list, const QString &substr, Qt::CaseSensitivity cs)
-{
-    return Utils::contains(list, [&substr, &cs](const QString &elem) {
-        return elem.contains(substr, cs);
-    });
+    setSourceModel(sourceModel);
+    setDynamicSortFilter(true);
+    setFilterCaseSensitivity(Qt::CaseInsensitive);
+    sort(0);
 }
 
 bool ExamplesListModelFilter::filterAcceptsRow(int sourceRow, const QModelIndex &sourceParent) const
 {
-    if (m_showTutorialsOnly) {
-        int type = sourceModel()->index(sourceRow, 0, sourceParent).data(Type).toInt();
-        if (type != Tutorial)
-            return false;
-    }
+    const ExampleItem item = sourceModel()->index(sourceRow, 0, sourceParent).data(
+                ExamplesListModel::ExampleItemRole).value<ExampleItem>();
 
-    if (!m_showTutorialsOnly) {
-        int type = sourceModel()->index(sourceRow, 0, sourceParent).data(Type).toInt();
-        if (type != Example && type != Demo)
-            return false;
-    }
+    if (m_showTutorialsOnly && item.type != Tutorial)
+        return false;
 
-    const QStringList tags = sourceModel()->index(sourceRow, 0, sourceParent).data(Tags).toStringList();
+    if (!m_showTutorialsOnly && item.type != Example && item.type != Demo)
+        return false;
 
     if (!m_filterTags.isEmpty()) {
-        return Utils::allOf(m_filterTags, [tags](const QString &filterTag) {
-            return tags.contains(filterTag);
+        return Utils::allOf(m_filterTags, [&item](const QString &filterTag) {
+            return item.tags.contains(filterTag);
         });
     }
 
-    if (!m_searchString.isEmpty()) {
-        const QString description = sourceModel()->index(sourceRow, 0, sourceParent).data(Description).toString();
-        const QString name = sourceModel()->index(sourceRow, 0, sourceParent).data(Name).toString();
-
-        foreach (const QString &subString, m_searchString) {
+    if (!m_filterStrings.isEmpty()) {
+        for (const QString &subString : m_filterStrings) {
             bool wordMatch = false;
-            wordMatch |= (bool)name.contains(subString, Qt::CaseInsensitive);
+            wordMatch |= bool(item.name.contains(subString, Qt::CaseInsensitive));
             if (wordMatch)
                 continue;
-            wordMatch |= containsSubString(tags, subString, Qt::CaseInsensitive);
+            const auto subMatch = [&subString](const QString &elem) { return elem.contains(subString); };
+            wordMatch |= Utils::contains(item.tags, subMatch);
             if (wordMatch)
                 continue;
-            wordMatch |= (bool)description.contains(subString, Qt::CaseInsensitive);
+            wordMatch |= bool(item.description.contains(subString, Qt::CaseInsensitive));
             if (!wordMatch)
                 return false;
         }
@@ -810,76 +744,6 @@ bool ExamplesListModelFilter::filterAcceptsRow(int sourceRow, const QModelIndex
     return QSortFilterProxyModel::filterAcceptsRow(sourceRow, sourceParent);
 }
 
-int ExamplesListModelFilter::rowCount(const QModelIndex &parent) const
-{
-    exampleDataRequested();
-    return QSortFilterProxyModel::rowCount(parent);
-}
-
-QVariant ExamplesListModelFilter::data(const QModelIndex &index, int role) const
-{
-    exampleDataRequested();
-    return QSortFilterProxyModel::data(index, role);
-}
-
-QAbstractItemModel* ExamplesListModelFilter::exampleSetModel()
-{
-    return m_sourceModel->exampleSetModel();
-}
-
-void ExamplesListModelFilter::filterForExampleSet(int index)
-{
-    if (m_blockIndexUpdate || !m_initalized)
-        return;
-
-    m_sourceModel->selectExampleSet(index);
-}
-
-void ExamplesListModelFilter::setShowTutorialsOnly(bool showTutorialsOnly)
-{
-    m_showTutorialsOnly = showTutorialsOnly;
-    emit showTutorialsOnlyChanged();
-}
-
-void ExamplesListModelFilter::handleQtVersionsChanged()
-{
-    m_blockIndexUpdate = true;
-    m_sourceModel->update();
-    m_blockIndexUpdate = false;
-}
-
-void ExamplesListModelFilter::qtVersionManagerLoaded()
-{
-    m_qtVersionManagerInitialized = true;
-    tryToInitialize();
-}
-
-void ExamplesListModelFilter::helpManagerInitialized()
-{
-    m_helpManagerInitialized = true;
-    tryToInitialize();
-}
-
-void ExamplesListModelFilter::exampleDataRequested() const
-{
-    ExamplesListModelFilter *that = const_cast<ExamplesListModelFilter *>(this);
-    that->m_exampleDataRequested = true;
-    that->tryToInitialize();
-}
-
-void ExamplesListModelFilter::tryToInitialize()
-{
-    if (!m_initalized
-            && m_qtVersionManagerInitialized && m_helpManagerInitialized && m_exampleDataRequested) {
-        m_initalized = true;
-        connect(QtVersionManager::instance(), &QtVersionManager::qtVersionsChanged,
-                this, &ExamplesListModelFilter::handleQtVersionsChanged);
-        connect(ProjectExplorer::KitManager::instance(), &ProjectExplorer::KitManager::defaultkitChanged,
-                this, &ExamplesListModelFilter::handleQtVersionsChanged);
-        handleQtVersionsChanged();
-    }
-}
-
 void ExamplesListModelFilter::delayedUpdateFilter()
 {
     if (m_timerId != 0)
@@ -888,15 +752,11 @@ void ExamplesListModelFilter::delayedUpdateFilter()
     m_timerId = startTimer(320);
 }
 
-int ExamplesListModelFilter::exampleSetIndex() const
-{
-    return m_sourceModel->selectedExampleSet();
-}
-
 void ExamplesListModelFilter::timerEvent(QTimerEvent *timerEvent)
 {
     if (m_timerId == timerEvent->timerId()) {
-        updateFilter();
+        invalidateFilter();
+        emit layoutChanged();
         killTimer(m_timerId);
         m_timerId = 0;
     }
@@ -984,31 +844,35 @@ struct SearchStringLexer
     }
 };
 
-void ExamplesListModelFilter::parseSearchString(const QString &arg)
+void ExamplesListModelFilter::setSearchString(const QString &arg)
 {
-    QStringList tags;
-    QStringList searchTerms;
+    if (m_searchString == arg)
+        return;
+
+    m_searchString = arg;
+    m_filterTags.clear();
+    m_filterStrings.clear();
+
+    // parse and update
     SearchStringLexer lex(arg);
     bool isTag = false;
     while (int tk = lex()) {
         if (tk == SearchStringLexer::TAG) {
             isTag = true;
-            searchTerms.append(lex.yytext);
+            m_filterStrings.append(lex.yytext);
         }
 
         if (tk == SearchStringLexer::STRING_LITERAL) {
             if (isTag) {
-                searchTerms.pop_back();
-                tags.append(lex.yytext);
+                m_filterStrings.pop_back();
+                m_filterTags.append(lex.yytext);
                 isTag = false;
             } else {
-                searchTerms.append(lex.yytext);
+                m_filterStrings.append(lex.yytext);
             }
         }
     }
 
-    setSearchStrings(searchTerms);
-    setFilterTags(tags);
     delayedUpdateFilter();
 }
 
