@@ -26,45 +26,36 @@
 #pragma once
 
 #include "texteditor_global.h"
-#include "helpitem.h"
 
-#include <QObject>
+#include <coreplugin/helpitem.h>
+#include <coreplugin/icontext.h>
 
-namespace Core { class IEditor; }
+#include <functional>
+
+QT_BEGIN_NAMESPACE
+class QPoint;
+QT_END_NAMESPACE
 
 namespace TextEditor {
 
-class BaseTextEditor;
 class TextEditorWidget;
 
-class TEXTEDITOR_EXPORT BaseHoverHandler : public QObject
+class TEXTEDITOR_EXPORT BaseHoverHandler
 {
-    Q_OBJECT
-
 public:
-    BaseHoverHandler();
-    ~BaseHoverHandler();
+    virtual ~BaseHoverHandler();
 
-    QString contextHelpId(TextEditorWidget *widget, int pos);
-    int checkToolTip(TextEditorWidget *widget, int pos);
-    void showToolTip(TextEditorWidget *widget, const QPoint &point, int pos);
+    void contextHelpId(TextEditorWidget *widget,
+                       int pos,
+                       const Core::IContext::HelpCallback &callback);
 
-    int priority() const;
+    using ReportPriority = std::function<void(int priority)>;
+    void checkPriority(TextEditorWidget *widget, int pos, ReportPriority report);
+    virtual void abort() {} // Implement for asynchronous priority reporter
+
+    void showToolTip(TextEditorWidget *widget, const QPoint &point, bool decorate = true);
 
 protected:
-    void setToolTip(const QString &tooltip);
-    void appendToolTip(const QString &extension);
-    const QString &toolTip() const;
-
-    void setIsDiagnosticTooltip(bool isDiagnosticTooltip);
-    bool isDiagnosticTooltip() const;
-
-    void setLastHelpItemIdentified(const HelpItem &help);
-    const HelpItem &lastHelpItemIdentified() const;
-
-    virtual void decorateToolTip();
-    virtual void operateTooltip(TextEditorWidget *editorWidget, const QPoint &point);
-
     enum {
         Priority_None = 0,
         Priority_Tooltip = 5,
@@ -72,16 +63,33 @@ protected:
         Priority_Diagnostic = 20
     };
     void setPriority(int priority);
+    int priority() const;
+
+    void setToolTip(const QString &tooltip);
+    const QString &toolTip() const;
+
+    void setLastHelpItemIdentified(const Core::HelpItem &help);
+    const Core::HelpItem &lastHelpItemIdentified() const;
+
+    bool isContextHelpRequest() const;
+
+    void propagateHelpId(TextEditorWidget *widget, const Core::IContext::HelpCallback &callback);
+
+    // identifyMatch() is required to report a priority by using the "report" callback.
+    // It is recommended to use e.g.
+    //    Utils::ExecuteOnDestruction reportPriority([this, report](){ report(priority()); });
+    // at the beginning of an implementation to ensure this in any case.
+    virtual void identifyMatch(TextEditorWidget *editorWidget, int pos, ReportPriority report);
+    virtual void decorateToolTip();
+    virtual void operateTooltip(TextEditorWidget *editorWidget, const QPoint &point);
+
 private:
-    void clear();
-    void process(TextEditorWidget *widget, int pos);
+    void process(TextEditorWidget *widget, int pos, ReportPriority report);
 
-    virtual void identifyMatch(TextEditorWidget *editorWidget, int pos);
-
-    bool m_diagnosticTooltip;
     QString m_toolTip;
-    HelpItem m_lastHelpItemIdentified;
-    int m_priority;
+    Core::HelpItem m_lastHelpItemIdentified;
+    int m_priority = -1;
+    bool m_isContextHelpRequest = false;
 };
 
 } // namespace TextEditor
