@@ -27,11 +27,13 @@
 
 #include "texteditor_global.h"
 #include "blockrange.h"
+#include "indenter.h"
 #include "codeassist/assistenums.h"
 
 #include <coreplugin/editormanager/editormanager.h>
 #include <coreplugin/editormanager/ieditor.h>
 #include <coreplugin/editormanager/ieditorfactory.h>
+#include <coreplugin/helpitem.h>
 
 #include <utils/link.h>
 #include <utils/uncommentselection.h>
@@ -57,6 +59,7 @@ class HighlightScrollBarController;
 
 namespace TextEditor {
 class TextDocument;
+class TextMark;
 class BaseHoverHandler;
 class RefactorOverlay;
 struct RefactorMarker;
@@ -66,6 +69,7 @@ class IAssistProvider;
 class ICodeStylePreferences;
 class CompletionAssistProvider;
 using RefactorMarkers = QList<RefactorMarker>;
+using TextMarks = QList<TextMark *>;
 
 namespace Internal {
 class BaseTextEditorPrivate;
@@ -85,7 +89,6 @@ class CompletionSettings;
 class DisplaySettings;
 class ExtraEncodingSettings;
 class FontSettings;
-class Indenter;
 class MarginSettings;
 class StorageSettings;
 class TypingSettings;
@@ -108,6 +111,7 @@ public:
     virtual void finalizeInitialization() {}
 
     static BaseTextEditor *currentTextEditor();
+    static QVector<BaseTextEditor *> textEditorsForDocument(TextDocument *textDocument);
 
     TextEditorWidget *editorWidget() const;
     TextDocument *textDocument() const;
@@ -121,7 +125,7 @@ public:
     void addContext(Core::Id id);
 
     // IEditor
-    Core::IDocument *document() override;
+    Core::IDocument *document() const override;
 
     IEditor *duplicate() override;
 
@@ -129,8 +133,8 @@ public:
     bool restoreState(const QByteArray &state) override;
     QWidget *toolBar() override;
 
-    void contextHelpId(const HelpIdCallback &callback) const override; // from IContext
-    void setContextHelpId(const QString &id) override;
+    void contextHelp(const HelpCallback &callback) const override; // from IContext
+    void setContextHelp(const Core::HelpItem &item) override;
 
     int currentLine() const override;
     int currentColumn() const override;
@@ -272,6 +276,9 @@ public:
     QRegion translatedLineRegion(int lineStart, int lineEnd) const;
 
     QPoint toolTipPosition(const QTextCursor &c) const;
+    void showTextMarksToolTip(const QPoint &pos,
+                              const TextMarks &marks,
+                              const TextMark *mainTextMark = nullptr) const;
 
     void invokeAssist(AssistKind assistKind, IAssistProvider *provider = nullptr);
 
@@ -325,7 +332,6 @@ public:
 
     // the blocks list must be sorted
     void setIfdefedOutBlocks(const QList<BlockRange> &blocks);
-    bool isMissingSyntaxDefinition() const;
 
     enum Side { Left, Right };
     QAction *insertExtraToolBarWidget(Side side, QWidget *widget);
@@ -442,6 +448,8 @@ public:
     void openLinkUnderCursor();
     void openLinkUnderCursorInNextSplit();
 
+    virtual void findUsages();
+
     /// Abort code assistant if it is running.
     void abortAssist();
 
@@ -472,6 +480,10 @@ signals:
 
     void requestBlockUpdate(const QTextBlock &);
 
+    void requestLinkAt(const QTextCursor &cursor, Utils::ProcessLinkCallback &callback,
+                       bool resolveTarget, bool inNextSplit);
+    void requestUsages(const QTextCursor &cursor);
+
 protected:
     QTextBlock blockForVisibleRow(int row) const;
     QTextBlock blockForVerticalOffset(int offset) const;
@@ -517,8 +529,6 @@ protected:
     virtual void triggerPendingUpdates();
     virtual void applyFontSettings();
 
-    virtual void onRefactorMarkerClicked(const RefactorMarker &) {}
-
     void showDefaultContextMenu(QContextMenuEvent *e, Core::Id menuContextId);
     virtual void finalizeInitialization() {}
     virtual void finalizeInitializationAfterDuplication(TextEditorWidget *) {}
@@ -537,8 +547,8 @@ public:
     QChar characterAt(int pos) const;
     QString textAt(int from, int to) const;
 
-    void contextHelpId(const Core::IContext::HelpIdCallback &callback);
-    void setContextHelpId(const QString &id);
+    void contextHelpItem(const Core::IContext::HelpCallback &callback);
+    void setContextHelpItem(const Core::HelpItem &item);
 
     static TextEditorWidget *currentTextEditorWidget();
 
@@ -628,7 +638,7 @@ public:
     using DocumentCreator = std::function<TextDocument *()>;
     using EditorWidgetCreator = std::function<TextEditorWidget *()>;
     using SyntaxHighLighterCreator = std::function<SyntaxHighlighter *()>;
-    using IndenterCreator = std::function<Indenter *()>;
+    using IndenterCreator = std::function<Indenter *(QTextDocument *)>;
     using AutoCompleterCreator = std::function<AutoCompleter *()>;
 
     void setDocumentCreator(const DocumentCreator &creator);
